SystemOrganization addCategory: #'OB-Standard-Browsers'!
SystemOrganization addCategory: #'OB-Standard-Commands'!
SystemOrganization addCategory: #'OB-Standard-Definitions'!
SystemOrganization addCategory: #'OB-Standard-Filters'!
SystemOrganization addCategory: #'OB-Standard-Nodes'!
SystemOrganization addCategory: #'OB-Standard-Panels'!
SystemOrganization addCategory: #'OB-Standard-Utilities'!

Notification subclass: #OBSourceFilesRequest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OB-Standard-Utilities'!

!OBSourceFilesRequest commentStamp: 'cwp 1/8/2005 13:09' prior: 0!
OBSourceFilesRequest is used to work around the fact that Squeak's source files are stored in a global array called SourceFiles. When testing OBMethodVersion and OBVersionBrowser, we don't want to use the real source files, as they are too unpredictable. Instead the test cases supply a source file array with known contents.

Therefore, OBMethodVersion never refers directly to the SourceFiles global. Instead it raises an OBSourceFiles request. If this occurs during a test, the test catches the notification and resumes using the test source files. Otherwise, the default action resumes using the global source file array.!

!OBSourceFilesRequest methodsFor: 'exceptionDescription' stamp: ''!
defaultAction
	^ SourceFiles! !

!OBSourceFilesRequest methodsFor: 'exceptionDescription' stamp: 'cwp 11/27/2004 22:49'!
isBrowseRequest
	^ false! !

OBFilter subclass: #OBClassSortFilter
	instanceVariableNames: 'indents order'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OB-Standard-Filters'!

!OBClassSortFilter methodsFor: 'private' stamp: 'cwp 5/20/2007 02:21'!
buildIndentIndex
	indents := IdentityDictionary new.
	order do: [:assoc | indents at: assoc value put: assoc key size - 1].! !

!OBClassSortFilter methodsFor: 'private' stamp: 'cwp 5/20/2007 02:20'!
collectSortedNodes
	^order asArray collect: [:assoc | assoc value]! !

!OBClassSortFilter methodsFor: 'filtering' stamp: 'cwp 5/20/2007 20:17'!
displayString: aString forParent: pNode child: cNode
	| count indented |
	count _ (indents at: cNode) * 2.
	indented _ String new: count + aString size.
	1 to: count do: [:i | indented at: i put: $ ].
	indented replaceFrom: count + 1 to: indented size with: aString.
	^ indented
! !

!OBClassSortFilter methodsFor: 'private' stamp: 'cwp 5/20/2007 02:20'!
initializeResults
	| i chain2 chain1 result max |
	order := SortedCollection sortBlock: 
					[:assoc1 :assoc2 | 
					result := nil.
					chain1 := assoc1 key.
					chain2 := assoc2 key.
					max := chain1 size min: chain2 size.
					i := 1.
					[result isNil and: [i <= max]] whileTrue: 
							[(chain1 at: i) = (chain2 at: i) 
								ifTrue: [i := i + 1]
								ifFalse: [result := (chain1 at: i) name < (chain2 at: i) name]].
					result isNil ifTrue: [chain1 size < chain2 size] ifFalse: [result]]! !

!OBClassSortFilter methodsFor: 'filtering' stamp: 'cwp 5/20/2007 20:41'!
nodesFrom: nodes forNode: parent 
	"Sort nodes according to their position in the class hierarchy"

	| classes supersChain |
	self initializeResults.
	classes := (nodes collect: [:classNode | classNode theClass]) 
				asOrderedCollection.
	nodes do: 
		[:classNode | 
		supersChain := classNode theClass withAllSuperclasses reversed.
		supersChain removeAllSuchThat: [:cl | (classes includes: cl) not].
		order add: supersChain contents -> classNode].
	self buildIndentIndex.
	^self collectSortedNodes! !

OBFilter subclass: #OBLimitProtocolFilter
	instanceVariableNames: 'theClass limitClass'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OB-Standard-Filters'!

!OBLimitProtocolFilter class methodsFor: 'instance creation' stamp: 'DM 5/23/2007 14:37'!
on: aClass
	^ self new forClass: aClass.! !

!OBLimitProtocolFilter methodsFor: 'filtering' stamp: 'DM 5/23/2007 01:36'!
defaultLimitClass
	"it's too bad that this needs to be duplicated between the ProtocolBrowser and here'"
	^ (theClass isKindOf: Metaclass) 
		ifTrue: [Object class]
		ifFalse: [Object]! !

!OBLimitProtocolFilter methodsFor: 'filtering' stamp: 'DM 5/22/2007 19:24'!
displayString: aString forNode: aNode
	(aNode isKindOf: OBMethodNode) ifFalse: [^ aString].
	^ aNode theClass = theClass
		ifTrue: [aString asText addAttribute: TextEmphasis bold]
		ifFalse: [aString].! !

!OBLimitProtocolFilter methodsFor: 'filtering' stamp: 'DM 5/23/2007 01:42'!
filterMethods: methods onClasses: allClasses forNode: aNode
	^ methods
		select: [:m | allClasses anySatisfy: [:c | c includesSelector: m selector]]! !

!OBLimitProtocolFilter methodsFor: 'filtering' stamp: 'DM 5/23/2007 01:44'!
filterProtocols: protocols onClasses: allClasses forNode: aNode 
	^ protocols allButFirst "the first is the synthetic --all-- category"
		select: [:p | allClasses anySatisfy: [:c | c organization categories includes: p name]]! !

!OBLimitProtocolFilter methodsFor: 'updating' stamp: 'DM 5/23/2007 14:37'!
forClass: aClass
	theClass := aClass! !

!OBLimitProtocolFilter methodsFor: 'filtering' stamp: 'DM 5/23/2007 01:17'!
limitClass: aClass
	limitClass := aClass.! !

!OBLimitProtocolFilter methodsFor: 'filtering' stamp: 'DM 5/23/2007 01:36'!
limitClassFor: aNode 
	^ limitClass ifNil: [self defaultLimitClass ] ifNotNil: [limitClass]! !

!OBLimitProtocolFilter methodsFor: 'filtering' stamp: 'DM 5/23/2007 01:43'!
nodesFrom: nodes forNode: aNode 
	| allClasses limit |
	nodes ifEmpty: [^ nodes].
	limit := self limitClassFor: aNode.
	allClasses := (aNode theClass withAllSuperclasses upTo: limit)
				add: limit; yourself.
	
	^ (nodes first isKindOf: OBMethodCategoryNode) 
		ifTrue: [self filterProtocols: nodes onClasses: allClasses forNode: aNode]
		ifFalse: [self filterMethods: nodes onClasses: allClasses forNode: aNode]! !

!OBLimitProtocolFilter methodsFor: 'filtering' stamp: 'DM 5/23/2007 14:35'!
setMetaNode: aMetaNode
	super setMetaNode: aMetaNode.
	aMetaNode children do: [ :each |
		(each filters includes: self)
			ifFalse: [ each addFilter: self ] ].! !

!ClassOrganizer methodsFor: '*ob-standard-testing' stamp: 'cwp 9/18/2004 23:59'!
isClassOrganizer
	^ true! !

OBNode subclass: #OBCodeNode
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OB-Standard-Nodes'!

!OBCodeNode commentStamp: 'cwp 1/8/2005 11:12' prior: 0!
OBCodeNode is an abstract superclass for node classes that represent program elements active in the image. Though it provides little functionality, it exists for structural purposes.!

OBCodeNode subclass: #OBClassAwareNode
	instanceVariableNames: 'theClass'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OB-Standard-Nodes'!

!OBClassAwareNode commentStamp: 'cwp 1/8/2005 11:23' prior: 0!
OBClassAware node models program elements that are part of a class. It provides methods for manipulating the class, as well as methods relating to sorting according to class hierarchy.

iVars

theClass 	- the class that this node is part of
superior 	- during hierarchical sorting this refers to the nearest superclass
			  that belongs to the group being sorted!

!OBClassAwareNode methodsFor: 'navigating' stamp: 'DM 5/31/2007 15:23'!
allCategories
	"Return all categories of theClass and its superclasses."
	| categories limit|
	limit := (self theClass = ProtoObject ) ifTrue: [ProtoObject] ifFalse: [Object].
	categories := Vocabulary fullVocabulary 
				categoryListForInstance: nil
				ofClass: self theClass
				limitClass: limit.
	^ (OrderedCollection new)
		add: (OBAllMethodCategoryNode on: self theClass);
		addAll: (categories 
					collect: [:cat | OBMethodCategoryNode on: cat inClass: self theClass]);
		yourself! !

!OBClassAwareNode methodsFor: 'actions' stamp: 'lr 3/6/2006 19:24'!
browse
	OBSystemBrowser openOnClass: self theNonMetaClass! !

!OBClassAwareNode methodsFor: 'nodes' stamp: 'dvf 8/31/2005 13:11'!
classNode
	^self theClass asNode! !

!OBClassAwareNode methodsFor: 'ancestry' stamp: ''!
isDescendantOfClass: aClassNode
	^ self theClassName = aClassNode theClassName! !

!OBClassAwareNode methodsFor: 'ancestry' stamp: 'cwp 12/5/2004 12:56'!
isDescendantOfClassCat: aClassCategoryNode

	"optimized version: sending #category to the class is slow"
	^ (self theNonMetaClass environment organization 
		listAtCategoryNamed: aClassCategoryNode name)
			includes: self theNonMetaClassName

! !

!OBClassAwareNode methodsFor: 'accessing' stamp: ''!
theClass
	^ theClass! !

!OBClassAwareNode methodsFor: 'accessing' stamp: ''!
theClass: aClass
	theClass := aClass! !

!OBClassAwareNode methodsFor: 'accessing' stamp: ''!
theClassName
	^ self theClass name! !

!OBClassAwareNode methodsFor: 'accessing' stamp: ''!
theMetaClass
	^ self theClass theMetaClass! !

!OBClassAwareNode methodsFor: 'accessing' stamp: ''!
theNonMetaClass
	^ self theClass theNonMetaClass! !

!OBClassAwareNode methodsFor: 'accessing' stamp: ''!
theNonMetaClassName
	^ self theNonMetaClass name! !

OBClassAwareNode subclass: #OBClassCommentNode
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OB-Standard-Nodes'!

!OBClassCommentNode commentStamp: 'cwp 1/8/2005 11:24' prior: 0!
OBClassCommentNode represents the comment attached to a particular class. !

!OBClassCommentNode class methodsFor: 'as yet unclassified' stamp: ''!
on: classRef
	^ self new setClass: classRef! !

!OBClassCommentNode methodsFor: 'public' stamp: 'cwp 12/13/2004 00:51'!
name
	^ self theClass name! !

!OBClassCommentNode methodsFor: 'initializing' stamp: ''!
setClass: aClass
	self theClass: aClass! !

!OBClassCommentNode methodsFor: 'definition' stamp: ''!
text
	^ self theClass comment! !

!OBClassCommentNode methodsFor: 'definition' stamp: ''!
text: aText
	self theClass comment: aText stamp: Utilities changeStamp.
	^ true! !

OBClassAwareNode subclass: #OBClassNode
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OB-Standard-Nodes'!

!OBClassNode commentStamp: 'cwp 1/8/2005 11:27' prior: 0!
OBClassNode wraps a class for display in various types of code browsers. It provides many navigation methods for the different types of browsers where classes may appear.!

!OBClassNode class methodsFor: 'instance creation' stamp: 'cwp 11/30/2004 21:06'!
on: aClass 
	^ self new setClass: aClass ! !

!OBClassNode methodsFor: 'accessing' stamp: 'cwp 12/12/2004 22:45'!
= other
	self species = other species ifFalse: [^ false].
	^ self theClass = other theClass! !

!OBClassNode methodsFor: 'navigating' stamp: 'cwp 1/7/2005 23:05'!
allCategory
	^ Array with: (OBAllMethodCategoryNode on: self theClass)! !

!OBClassNode methodsFor: 'ancestry' stamp: ''!
ancestrySelector
	^ #isDescendantOfClass:! !

!OBClassNode methodsFor: 'navigating' stamp: 'cwp 1/7/2005 23:06'!
categories
	^ self theClass organization categories
		collect: [:cat | OBMethodCategoryNode on: cat inClass: self theClass]
			! !

!OBClassNode methodsFor: 'nodes' stamp: 'cwp 9/20/2005 08:42'!
categoryNodeNamed: aString 
	^ OBMethodCategoryNode on: aString inClass: theClass! !

!OBClassNode methodsFor: 'nodes' stamp: ''!
classCategoryNode
	^ OBClassCategoryNode 
		on: self theNonMetaClass category
		inEnvironment: self theClass environment! !

!OBClassNode methodsFor: 'navigating' stamp: 'cwp 12/12/2004 23:31'!
classHierarchy
	^ self nodeHierarchyWithClass: OBClassNode! !

!OBClassNode methodsFor: 'navigating' stamp: ''!
classVariables
	^ self theClass allClassVarNames asArray sort
		collect: [:ea | OBClassVariableNode on: ea inClass: self theClass]! !

!OBClassNode methodsFor: 'navigating' stamp: 'cwp 12/12/2004 23:31'!
commentHierarchy
	^ self nodeHierarchyWithClass: OBClassCommentNode! !

!OBClassNode methodsFor: 'accessing' stamp: ''!
definition
	^ OBClassDefinition 
		environment: self theClass environment 
		template: self theClass definition! !

!OBClassNode methodsFor: 'drag and drop' stamp: 'lr 3/28/2006 18:29'!
dropOnClassCategory: aNode
	self theNonMetaClass category: aNode name.
	self signalSelection.
	^ true! !

!OBClassNode methodsFor: 'drag and drop' stamp: 'lr 4/6/2006 11:31'!
dropSelector
	^ #dropOnClass:! !

!OBClassNode methodsFor: 'commands' stamp: 'cwp 10/26/2004 00:49'!
fileOut
	self theNonMetaClass fileOut! !

!OBClassNode methodsFor: 'testing' stamp: 'cwp 6/30/2006 00:57'!
hasOrganization
	^ true! !

!OBClassNode methodsFor: 'navigating' stamp: ''!
instanceVariables
	^ self theClass allInstVarNames asArray sort
		collect: [:ea | OBInstanceVariableNode on: ea inClass: self theClass]! !

!OBClassNode methodsFor: 'ancestry' stamp: 'DM 5/20/2007 15:40'!
isDescendantOfMethodCat: category
	^ false.! !

!OBClassNode methodsFor: 'navigating' stamp: 'cwp 12/12/2004 23:30'!
metaclassHierarchy
	^ self nodeHierarchyWithClass: OBMetaclassNode! !

!OBClassNode methodsFor: 'displaying' stamp: 'cwp 12/13/2004 00:51'!
name
	^ self theClass name! !

!OBClassNode methodsFor: 'navigating' stamp: 'cwp 5/20/2007 00:27'!
nodeHierarchyWithClass: aClass
	^ self surroundingHierarchy collect: [:ea | aClass on: ea]! !

!OBClassNode methodsFor: 'accessing' stamp: ''!
organization
	^ self theClass organization! !

!OBClassNode methodsFor: 'printing' stamp: 'cwp 12/10/2004 23:53'!
printOn: aStream
	aStream
		print: self class;
		nextPut: $<;
		print: self theClass;
		nextPut: $>! !

!OBClassNode methodsFor: 'initializing' stamp: ''!
setClass: aClass
	self theClass: aClass! !

!OBClassNode methodsFor: 'accessing' stamp: 'cwp 12/12/2004 23:27'!
surroundingHierarchy
	 ^ (self theClass withAllSuperclasses union: self theClass allSubclasses)
		asArray! !

!OBClassNode methodsFor: 'displaying' stamp: ''!
title
	^ self name! !

!OBClassNode methodsFor: 'navigating' stamp: 'cwp 12/8/2004 23:45'!
users
	^ (SystemNavigation default allCallsOn: (theClass environment associationAt: theClass name))
		collect: [:ref | OBClassRefNode on: self name inMethod: ref]! !

OBClassNode subclass: #OBMetaclassNode
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OB-Standard-Nodes'!

!OBMetaclassNode commentStamp: 'cwp 1/8/2005 11:29' prior: 0!
OBMetaclassNode is essentially the same as an OBClassNode, but overrides a few methods to work properly with metaclasses.!

!OBMetaclassNode methodsFor: 'actions' stamp: 'cwp 12/15/2004 21:57'!
browse
	^ OBSystemBrowser openOnClass: self theNonMetaClass ! !

!OBMetaclassNode methodsFor: 'displaying' stamp: 'cwp 12/13/2004 00:56'!
name 
	^ self nonMetaName! !

!OBMetaclassNode methodsFor: 'displaying' stamp: ''!
nonMetaName
	^ self theNonMetaClass name! !

!OBMetaclassNode methodsFor: 'initializing' stamp: ''!
setClass: aClass
	super setClass: aClass theMetaClass
		! !

OBClassAwareNode subclass: #OBMethodCategoryNode
	instanceVariableNames: 'name'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OB-Standard-Nodes'!

!OBMethodCategoryNode commentStamp: 'cwp 1/8/2005 12:29' prior: 0!
OBMethodCategory represents a category within a ClassOrganization. Instead of an organization definition, OBMethodCategory presents a MethodDefinition with the class' default method template.!

OBMethodCategoryNode subclass: #OBAllMethodCategoryNode
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OB-Standard-Nodes'!

!OBAllMethodCategoryNode commentStamp: 'cwp 1/8/2005 12:31' prior: 0!
OBAllMethodCategory implements the synthetic '-- all --' category, which contains all the methods in a class.!

!OBAllMethodCategoryNode class methodsFor: 'as yet unclassified' stamp: ''!
on: className
	^ self on: '-- all --' inClass: className! !

!OBAllMethodCategoryNode methodsFor: 'accessing' stamp: ''!
category
	^ 'as yet unclassified'! !

!OBAllMethodCategoryNode methodsFor: 'accessing' stamp: ''!
methodReferences
	^ self theClass selectors asSortedArray 
		collect: [:ea | MethodReference new setStandardClass: self theClass methodSymbol: ea]! !

!OBMethodCategoryNode class methodsFor: 'actions' stamp: 'cwp 9/18/2004 22:58'!
actionsForParent: aNode
	^ #()! !

!OBMethodCategoryNode class methodsFor: 'instance creation' stamp: ''!
on: aString inClass: aClassReference
	^ self new setName: aString class: aClassReference! !

!OBMethodCategoryNode methodsFor: 'comparing' stamp: 'cwp 12/20/2004 00:50'!
= other
	self species = other species ifFalse: [^ false].	
	self theClass = other theClass ifFalse: [^ false].
	^ self name = other name! !

!OBMethodCategoryNode methodsFor: 'navigating' stamp: 'DM 5/20/2007 14:08'!
allMethods
	| allMethods |
	allMethods := Vocabulary fullVocabulary 
					allMethodsInCategory: self name
					forInstance: nil
					ofClass: self theClass.
	^ allMethods collect: [:selector | | class |
					class := self theClass whichClassIncludesSelector:  selector.
					OBMethodNode on: selector inClass: class].
					! !

!OBMethodCategoryNode methodsFor: 'ancestry' stamp: ''!
ancestrySelector
	^ #isDescendantOfMethodCat:! !

!OBMethodCategoryNode methodsFor: 'actions' stamp: 'cwp 12/15/2004 21:50'!
browse
	OBSystemBrowser openOnClass: self theClass category: name! !

!OBMethodCategoryNode methodsFor: 'accessing' stamp: ''!
category
	^ name! !

!OBMethodCategoryNode methodsFor: 'accessing' stamp: ''!
container
	^ self theClass! !

!OBMethodCategoryNode methodsFor: 'public' stamp: ''!
definition
	^ OBMethodDefinition inCategory: self category inClass: self theClass! !

!OBMethodCategoryNode methodsFor: 'drag and drop' stamp: 'lr 4/6/2006 11:46'!
dropOnClass: aNode
	self methods do: [ :each | each dropOnClass: aNode ].
	aNode signalChildrenChanged.
	^ true! !

!OBMethodCategoryNode methodsFor: 'drag and drop' stamp: ''!
dropSelector
	^ #dropOnMethodCategory:! !

!OBMethodCategoryNode methodsFor: 'services' stamp: ''!
fileOut
	self theClass fileOutCategory: name! !

!OBMethodCategoryNode methodsFor: 'comparing' stamp: 'cwp 12/20/2004 00:53'!
hash
	^ self name hash! !

!OBMethodCategoryNode methodsFor: 'testing' stamp: 'cwp 7/17/2006 08:48'!
isCategory
	^ true! !

!OBMethodCategoryNode methodsFor: 'ancestry' stamp: ''!
isDescendantOfMethodCat: other
	^ (other theClassName = self theClassName) and: [other name = name]! !

!OBMethodCategoryNode methodsFor: 'accessing' stamp: 'cwp 9/14/2005 09:06'!
methodReferences
	^ (self theClass organization listAtCategoryNamed: name)
		collect: [:ea | MethodReference new
						setClassSymbol: self theNonMetaClassName
						classIsMeta: self theClass isMeta
						methodSymbol: ea
						stringVersion: '']
! !

!OBMethodCategoryNode methodsFor: 'navigating' stamp: 'dvf 8/19/2005 17:16'!
methods
	^ self methodReferences collect: [:ref | ref asNode]! !

!OBMethodCategoryNode methodsFor: 'public' stamp: ''!
name
	^ name! !

!OBMethodCategoryNode methodsFor: 'accessing' stamp: 'dvf 8/16/2005 17:47'!
printOn: aStream
	super printOn: aStream.
	aStream nextPut: $<.
	self name printOn: aStream.
	aStream nextPut: $>.! !

!OBMethodCategoryNode methodsFor: 'actions' stamp: 'cwp 9/19/2004 00:11'!
remove
	self theClass removeCategory: name.
	self signalDeletion! !

!OBMethodCategoryNode methodsFor: 'initializing' stamp: ''!
setName: aString class: aClass
	name := aString.
	self theClass: aClass! !

OBClassAwareNode subclass: #OBMethodNode
	instanceVariableNames: 'selector'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OB-Standard-Nodes'!

!OBMethodNode commentStamp: 'cwp 1/8/2005 12:37' prior: 0!
OBMethodNode wraps a method in a particular class. It supplies an OBMethodDefinition for displaying and editing the source code of the method, and various actions for manipulating the method. It has two roles: first it appears in the right most pane of a standard system browser. Second, it often serves as the root node for an OBListBrowser, and has navigation methods for senders, implementors etc.

OBMethodNode also serves as the superclass for nodes that represent *parts* of a method, such as sends of a particular message, references to a class, accesses to a instance variable, etc.!

OBMethodNode subclass: #OBClassRefNode
	instanceVariableNames: 'className'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OB-Standard-Nodes'!

!OBClassRefNode commentStamp: 'cwp 1/8/2005 12:38' prior: 0!
Each instance of OBClassRefNode refers to a reference to a particular class from the source code of a method. It's used in the OBListBrowser created by the 'browse references' class action.!

!OBClassRefNode class methodsFor: 'as yet unclassified' stamp: 'cwp 12/1/2004 00:50'!
on: aClassName inMethod: aMethodRef
	^ self new setClassName: aClassName reference: aMethodRef! !

!OBClassRefNode methodsFor: 'accessing' stamp: 'cwp 12/8/2004 22:14'!
name
	^ className! !

!OBClassRefNode methodsFor: 'accessing' stamp: 'cwp 12/1/2004 01:16'!
selection
	| start parser |
	(parser _ Compiler parserClass new) parseSelector: self source.
	start := parser endOfLastToken.
	start := (self source asString findString: className startingAt: start).
	^ start to: start + className size - 1! !

!OBClassRefNode methodsFor: 'initialize-release' stamp: 'cwp 12/1/2004 01:10'!
setClassName: theName reference: aMethodRef
	self setReference: aMethodRef.
	className := theName.
! !

OBMethodNode subclass: #OBMessageNode
	instanceVariableNames: 'message'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OB-Standard-Nodes'!

!OBMessageNode commentStamp: 'cwp 1/8/2005 12:40' prior: 0!
An OBMessageNode refers to a message send in the source code of a method. They are typically used in the 'senders' browser.!

!OBMessageNode class methodsFor: 'instance creation' stamp: ''!
fromMethodNode: aMethodNode
	^ self new 
		setMessage: aMethodNode selector
		selector: aMethodNode selector
		class: aMethodNode theClass! !

!OBMessageNode class methodsFor: 'instance creation' stamp: ''!
on: aSelector inMethod: aSelector2 inClass: classRef
	^ self new setMessage: aSelector selector: aSelector2 class: classRef! !

!OBMessageNode class methodsFor: 'instance creation' stamp: ''!
on: aSelector inMethodNode: aNode
	^ self 
		on: aSelector 
		inMethod: aNode selector 
		inClass: aNode theClass! !

!OBMessageNode class methodsFor: 'instance creation' stamp: 'cwp 12/8/2004 23:46'!
on: aSelector inMethodReference: ref
	^ self 
		on: aSelector 
		inMethod: ref methodSymbol
		inClass: ref actualClass! !

!OBMessageNode methodsFor: 'navigating' stamp: ''!
implementors
	^ self implementorsOf: message! !

!OBMessageNode methodsFor: 'displaying' stamp: 'cwp 12/8/2004 21:26'!
name
	^ message! !

!OBMessageNode methodsFor: 'accessing' stamp: ''!
selection
	| methodNode assoc |
	methodNode := Parser new 
					parse: self source
					class: self theClass.
	assoc := (methodNode encoder rawSourceRanges) 
			associations detect: [:ea | ea key isMessage: message receiver: nil arguments: nil]
						ifNone: [nil -> (1 to: 0)].
						"Some messages are generated by the compiler."
	^ assoc value! !

!OBMessageNode methodsFor: 'navigating' stamp: ''!
sendersOfMessage
	^ self sendersOf: message! !

!OBMessageNode methodsFor: 'initializing' stamp: ''!
setMessage: aSelector selector: aSelector2 class: aClass
	message := aSelector.
	selector := aSelector2.
	self theClass: aClass! !

!OBMethodNode class methodsFor: 'instance creation' stamp: ''!
on: aMethodReference
	^ self new setReference: aMethodReference! !

!OBMethodNode class methodsFor: 'instance creation' stamp: ''!
on: aSelector inClass: aClassReference
	^ self new setSelector: aSelector class: aClassReference! !

!OBMethodNode methodsFor: 'comparing' stamp: 'cwp 12/8/2004 22:07'!
= other
	^ (self species = other species)
		and: [self theClass = other theClass] 
		and: [self selector = other selector]! !

!OBMethodNode methodsFor: 'private' stamp: ''!
addOverridersOf: aSelector inClass: aClass to: aCollection
	aClass subclasses do: 
		[:ea | (ea includesSelector: aSelector)
					ifTrue: [aCollection add: ea]
					ifFalse: [self addOverridersOf: aSelector inClass: ea to: aCollection]]! !

!OBMethodNode methodsFor: 'ancestry' stamp: ''!
ancestrySelector
	^ #isDescendantOfMethod: ! !

!OBMethodNode methodsFor: 'displaying' stamp: 'dc 5/2/2007 19:05'!
annotationString
	^ (OBAnnotationRequest onClass: self theClass selector: self selector)
		getAnnotations.! !

!OBMethodNode methodsFor: 'actions' stamp: 'cwp 12/5/2004 18:35'!
browse
	OBSystemBrowser openOnClass: self theClass selector: selector! !

!OBMethodNode methodsFor: 'accessing' stamp: 'cwp 12/1/2004 00:36'!
definition
	^ OBMethodDefinition
		selection: self selection
		source: self source
		inClass: self theClass! !

!OBMethodNode methodsFor: 'displaying' stamp: 'lr 3/29/2006 12:08'!
displayString
	^ (self theClass respondsTo: #includesLocalSelector:) 
		ifFalse: [ super displayString ]
		ifTrue: [
			(self theClass includesLocalSelector: selector)
				ifTrue: [ super displayString  ]
				ifFalse: [ 
					super displayString asText 
						addAttribute: TextEmphasis italic ] ]! !

!OBMethodNode methodsFor: 'private' stamp: 'cwp 10/18/2004 00:29'!
doRemove
	self theClass removeSelector: self selector.
	self signalDeletion! !

!OBMethodNode methodsFor: 'drag and drop' stamp: 'lr 4/18/2006 08:55'!
dropOnClass: aNode
	aNode theClass 
		compile: self source
		classified: (self theClass organization 
			categoryOfElement: self selector).
	InputSensor default shiftPressed
		ifFalse: [ self theClass removeSelector: self selector ].
	aNode signalChildrenChanged.
	^ true! !

!OBMethodNode methodsFor: 'drag and drop' stamp: 'cwp 9/18/2005 19:01'!
dropOnMethodCategory: aNode

	"We don't support dropping on another class yet."
	aNode theClass = self theClass ifFalse: [^ false].
	
	self theClass organization classify: self selector under: aNode name.
	aNode signalChildrenChanged.
	
	^ true! !

!OBMethodNode methodsFor: 'actions' stamp: ''!
fileOut
	self theClass fileOutMethod: selector! !

!OBMethodNode methodsFor: 'displaying' stamp: 'cwp 12/8/2004 21:08'!
fullName
	^ self theClassName, '>>', selector! !

!OBMethodNode methodsFor: 'testing' stamp: 'cwp 10/11/2006 20:46'!
hasSelector
	^ true! !

!OBMethodNode methodsFor: 'testing' stamp: 'cwp 10/9/2006 18:13'!
hasVersions
	^ true! !

!OBMethodNode methodsFor: 'comparing' stamp: 'cwp 12/8/2004 22:07'!
hash
	^ theClass hash bitXor: selector hash! !

!OBMethodNode methodsFor: 'navigating' stamp: 'cwp 1/2/2005 23:12'!
implementors
	^ self implementorsOf: selector! !

!OBMethodNode methodsFor: 'private' stamp: 'cwp 12/8/2004 23:48'!
implementorsOf: aSelector
	^ (SystemNavigation default allImplementorsOf: aSelector) asSortedArray
			collect: [:ref | OBMethodNode on: ref]! !

!OBMethodNode methodsFor: 'private' stamp: 'cwp 12/8/2004 23:49'!
inheritanceRoot
	| rootClass |
	rootClass := (self theClass withAllSuperclasses asArray
					select: [:ea | ea includesSelector: self selector]) last.
	^ OBCollectionNode on: {OBMethodNode 
								on: self selector 
								inClass: rootClass}! !

!OBMethodNode methodsFor: 'ancestry' stamp: 'cwp 11/28/2004 10:37'!
isDescendantOfMethod: other
	^ other selector = selector
		and: [self theClass withAllSuperclasses includes: other theClass].
	! !

!OBMethodNode methodsFor: 'ancestry' stamp: 'cwp 12/2/2004 23:28'!
isDescendantOfMethodCat: aMethodCatNode
	^ (self theClass organization categoryOfElement: self selector) = aMethodCatNode name! !

!OBMethodNode methodsFor: 'ancestry' stamp: 'cwp 10/17/2004 23:55'!
isDescendantOfMethodVersion: anOBMethodVersionNode 
	^ false! !

!OBMethodNode methodsFor: 'nodes' stamp: ''!
messageNode
	^ OBMessageNode fromMethodNode: self! !

!OBMethodNode methodsFor: 'private' stamp: ''!
messageSelectors
	^ ((self theClass compiledMethodAt: self selector ifAbsent: [^ #()]) messages) asSortedArray! !

!OBMethodNode methodsFor: 'navigating' stamp: ''!
messages
	^ (((self theClass
				compiledMethodAt: self selector 
				ifAbsent: [^ #()]) messages) asSortedArray)
					collect: [:sel | OBMessageNode 
									on: sel 
									inMethod: selector 
									inClass: self theClass]! !

!OBMethodNode methodsFor: 'actions' stamp: 'cwp 7/11/2006 02:43'!
moveToPackage
	| packagesNames selection packages |
	packages := PackageOrganizer default packages 
					asSortedCollection: [:a :b | a packageName <= b packageName].
	packagesNames := packages collect: [:ea | ea packageName].
	selection := OBChoiceRequest prompt: nil labels: packagesNames values: packages.
	selection ifNotNil: [selection addMethod: self reference].! !

!OBMethodNode methodsFor: 'displaying' stamp: 'cwp 12/8/2004 21:08'!
name
	^ selector! !

!OBMethodNode methodsFor: 'displaying' stamp: 'jk 5/20/2007 11:01'!
nameWithClassNameInBrackets
	^ selector, '  (', self theClassName, ')'! !

!OBMethodNode methodsFor: 'navigating' stamp: 'cwp 12/8/2004 23:49'!
overrides
	| classes |
	classes := OrderedCollection new.
	self addOverridersOf: self selector inClass: self theClass to: classes.
	^ classes collect: [:ea | OBMethodNode on: selector inClass: ea] ! !

!OBMethodNode methodsFor: 'printing' stamp: 'cwp 12/8/2004 21:59'!
printOn: aStream
	aStream
		nextPutAll: self class name;
		nextPut: $<;
		nextPutAll: self theClass name;
		nextPut: $#;
		nextPutAll: self selector;
		nextPut: $>! !

!OBMethodNode methodsFor: 'accessing' stamp: ''!
reference
	^ self referenceForMethod: selector ofClass: self theClassName! !

!OBMethodNode methodsFor: 'actions' stamp: 'cwp 10/18/2004 00:30'!
remove
	| senders choice |
	senders _ SystemNavigation default allCallsOn: self selector.
	senders isEmpty ifTrue: [^ self doRemove].
	choice _ OBChoiceRequest
				prompt: 'This message has ', senders size asString, ' senders.'
				labels: #('Remove it' 
						'Remove, then browse senders' 
						'Don''t remove, but show me those senders' 
						'Forget it -- do nothing -- sorry I asked')
				values: #(doRemove removeAndBrowse simpleBrowseSenders nil).
	choice ifNotNil: [^ self perform: choice]
! !

!OBMethodNode methodsFor: 'private' stamp: ''!
removeAndBrowse
	self simpleBrowseSenders.
	self theClass removeSelector: self selector.
	self signalDeletion! !

!OBMethodNode methodsFor: 'accessing' stamp: 'cwp 12/1/2004 00:37'!
selection
	^ 1 to: 0! !

!OBMethodNode methodsFor: 'accessing' stamp: ''!
selector
	^ selector! !

!OBMethodNode methodsFor: 'navigating' stamp: ''!
selectorAndMessages
	^ self messages copyWithFirst: self messageNode! !

!OBMethodNode methodsFor: 'navigating' stamp: ''!
senders
	^ self sendersOf: selector! !

!OBMethodNode methodsFor: 'private' stamp: ''!
sendersOf: aSelector
	^ (SystemNavigation default allCallsOn: aSelector) asSortedArray
			collect: [:ref | OBMessageNode on: aSelector inMethodReference: ref]! !

!OBMethodNode methodsFor: 'initializing' stamp: ''!
setReference: aMethodReference
	self 
		setSelector: aMethodReference methodSymbol
		class: (aMethodReference actualClass)! !

!OBMethodNode methodsFor: 'initializing' stamp: ''!
setSelector: aSelector class: aClass
	selector := aSelector.
	self theClass: aClass
! !

!OBMethodNode methodsFor: 'private' stamp: 'avi 9/17/2005 01:29'!
simpleBrowseSenders
	OBSendersBrowser browseRoot: self! !

!OBMethodNode methodsFor: 'actions' stamp: 'cwp 12/1/2004 00:36'!
source
	^ (self theClass sourceCodeAt: self selector ifAbsent: [^ '']) 
		asText makeSelectorBold! !

!OBMethodNode methodsFor: 'accessing' stamp: ''!
sourceFiles
	^ OBSourceFilesRequest signal! !

!OBMethodNode methodsFor: 'private' stamp: ''!
sourcePointer
	^ (self theClass compiledMethodAt: self selector) sourcePointer! !

!OBMethodNode methodsFor: 'navigating' stamp: ''!
versions
	^ (OBMethodVersion scan: self sourceFiles from: self sourcePointer)
		collect: [:ea | OBMethodVersionNode on: ea]! !

OBMethodNode subclass: #OBMethodVersionNode
	instanceVariableNames: 'version'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OB-Standard-Nodes'!

!OBMethodVersionNode commentStamp: 'cwp 1/8/2005 12:41' prior: 0!
OBMethodVersions refer to OBMethodVersions, and are used by the VersionBrowser.!

!OBMethodVersionNode class methodsFor: 'instance creation' stamp: ''!
on: aMethodVersion 
	^ self new setVersion: aMethodVersion! !

!OBMethodVersionNode methodsFor: 'actions' stamp: 'cwp 12/15/2004 21:51'!
browse
	OBSystemBrowser openOnClass: self theClass selector: version selector! !

!OBMethodVersionNode methodsFor: 'accessing' stamp: 'cwp 10/17/2004 23:57'!
category
	^ self theClass organization categoryOfElement: version selector! !

!OBMethodVersionNode methodsFor: 'compiling' stamp: 'cwp 10/17/2004 23:53'!
definition
	^ (OBMethodDefinition source: version source inClass: self theClass)
		callback: [:sel | 
					version selector = sel
						ifTrue: [self class on: version latest]
						ifFalse: [OBMethodNode on: sel inClass: self theClass]]! !

!OBMethodVersionNode methodsFor: 'testing' stamp: 'cwp 10/11/2006 20:54'!
hasSelector
	^ true! !

!OBMethodVersionNode methodsFor: 'ancestry' stamp: 'cwp 10/17/2004 23:43'!
isDescendantOfMethodVersion: aNode
	^ (version = aNode version)! !

!OBMethodVersionNode methodsFor: 'displaying' stamp: 'cwp 11/27/2004 21:38'!
name
	| stamp |
	stamp := version stamp ifNil: ['<timestamp missing>'].
	^ version theClassName, '>>', version selector, ' ', stamp! !

!OBMethodVersionNode methodsFor: 'accessing' stamp: ''!
reference
	^ self referenceForMethod: version selector ofClass: self theClassName.! !

!OBMethodVersionNode methodsFor: 'accessing' stamp: 'cwp 10/11/2006 20:55'!
selector
	^ version selector! !

!OBMethodVersionNode methodsFor: 'initializing' stamp: 'cwp 3/11/2007 21:29'!
setVersion: aMethodVersion
	version := aMethodVersion.
	self setSelector: version selector class: version theClass! !

!OBMethodVersionNode methodsFor: 'accessing' stamp: 'cwp 10/17/2004 23:44'!
version
	^ version! !

OBClassAwareNode subclass: #OBVariableNode
	instanceVariableNames: 'name'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OB-Standard-Nodes'!

!OBVariableNode commentStamp: 'cwp 1/8/2005 12:46' prior: 0!
OBVariableNode is an abstract superclass for the two types of variables a class can contain - class variables and instance variables. The only difference between the two subclasses is how they search for methods that refer to them.!

OBVariableNode subclass: #OBClassVariableNode
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OB-Standard-Nodes'!

!OBClassVariableNode commentStamp: 'cwp 1/8/2005 12:51' prior: 0!
OBClassVariable provides a method for finding methods which refer to a shared variable, ie, by searching for an association in the literal frame rather than for bytecodes refering to an instance variable.!

!OBClassVariableNode methodsFor: 'navigating' stamp: 'cwp 12/8/2004 23:47'!
accessors
	| literal |
	literal := (self theClass withAllSuperclasses 
				gather: [:ea | ea classPool associations])
					detect: [:ea | ea key = name].
	^ ((self systemNavigation allCallsOn: literal) asArray sort)
		collect: [:ref | OBMethodNode on: ref]! !

OBVariableNode subclass: #OBInstanceVariableNode
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OB-Standard-Nodes'!

!OBInstanceVariableNode commentStamp: 'cwp 1/8/2005 12:50' prior: 0!
OBClassVariable provides a method for finding methods which refer to an instance, ie, by searching for instance variable access bytecodes rather than for an association in the literal frame.!

!OBInstanceVariableNode methodsFor: 'navigating' stamp: 'cwp 12/8/2004 23:48'!
accessors
	| accessors |
	accessors := OrderedCollection new.
	self theClass withAllSubAndSuperclassesDo: [:class |
		(class whichSelectorsAccess: name) asSortedCollection
			do: [:ea | ea = #DoIt ifFalse: [accessors add: 
					(self referenceForMethod: ea ofClass: class name)]]].
	^ accessors asArray	collect: [:ref | OBMethodNode on: ref]! !

!OBVariableNode class methodsFor: 'instance creation' stamp: ''!
on: instVarName inClass: aClassReference 
	^ self new setName: instVarName class: aClassReference! !

!OBVariableNode methodsFor: 'displaying' stamp: ''!
name
	^ name! !

!OBVariableNode methodsFor: 'initializing' stamp: ''!
setName: aString class: aClass
	name := aString.
	self theClass: aClass! !

OBCodeNode subclass: #OBClassCategoryNode
	instanceVariableNames: 'environment name'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OB-Standard-Nodes'!

!OBClassCategoryNode commentStamp: 'cwp 1/8/2005 12:58' prior: 0!
OBClassCategory represents a system category in the image's SystemOrganization.!

!OBClassCategoryNode class methodsFor: 'instance creation' stamp: ''!
on: aString
	^ self new setName: aString! !

!OBClassCategoryNode class methodsFor: 'instance creation' stamp: ''!
on: aString inEnvironment: anEnvironment
	^ self new setName: aString environment: anEnvironment! !

!OBClassCategoryNode methodsFor: 'ancestry' stamp: ''!
ancestrySelector
	^ #isDescendantOfClassCat:! !

!OBClassCategoryNode methodsFor: 'actions' stamp: 'cwp 12/15/2004 21:44'!
browse
	OBSystemBrowser openOnEnvironment: environment category: name! !

!OBClassCategoryNode methodsFor: 'private' stamp: ''!
classNames
	^ environment organization listAtCategoryNamed: name! !

!OBClassCategoryNode methodsFor: 'navigating' stamp: 'dvf 8/15/2005 17:31'!
classes
	^ self classNames collect: [:ea | (environment at: ea) asNode]! !

!OBClassCategoryNode methodsFor: 'navigating' stamp: 'dvf 8/17/2005 17:25'!
comments
	^ self classNames collect: [:ea | (environment at: ea) asCommentNode ]! !

!OBClassCategoryNode methodsFor: 'accessing' stamp: 'cwp 9/22/2004 22:13'!
container
	^ environment! !

!OBClassCategoryNode methodsFor: 'displaying' stamp: ''!
definition
	^ OBClassDefinition 
		environment: environment 
		template: ((environment at: #Class) template: name)! !

!OBClassCategoryNode methodsFor: 'drag and drop' stamp: ''!
dropSelector
	^ #dropOnClassCategory:! !

!OBClassCategoryNode methodsFor: 'services' stamp: ''!
fileOut
	environment organization fileOutCategory: name.! !

!OBClassCategoryNode methodsFor: 'testing' stamp: 'cwp 7/17/2006 08:47'!
isCategory
	^ true! !

!OBClassCategoryNode methodsFor: 'ancestry' stamp: ''!
isDescendantOfClassCat: other
	^ other name = name! !

!OBClassCategoryNode methodsFor: 'class creation' stamp: ''!
isRedefinedBy: definition
	"No class is selected, so the definition can't be a redefinition."
	
	^ false! !

!OBClassCategoryNode methodsFor: 'navigating' stamp: 'dvf 8/15/2005 17:52'!
metaclasses
	^self classNames collect: [:ea | (environment at: ea) asClassSideNode]! !

!OBClassCategoryNode methodsFor: 'displaying' stamp: ''!
name
	^ name! !

!OBClassCategoryNode methodsFor: 'printing' stamp: 'cwp 5/11/2007 00:59'!
printOn: aStream
	aStream
		nextPutAll: self class name;
		nextPut: $<;
		nextPutAll: self name;
		nextPut: $>! !

!OBClassCategoryNode methodsFor: 'actions' stamp: ''!
printOut
	environment organization fileOutCategory: name asHtml: true! !

!OBClassCategoryNode methodsFor: 'actions' stamp: 'cwp 9/19/2004 00:11'!
remove
	environment organization removeSystemCategory: name.
	self signalDeletion! !

!OBClassCategoryNode methodsFor: 'initializing' stamp: ''!
setName: aString
	self setName: aString environment: self class environment! !

!OBClassCategoryNode methodsFor: 'initializing' stamp: ''!
setName: aString environment: anEnvironment
	name := aString asSymbol.
	environment := anEnvironment! !

!OBClassCategoryNode methodsFor: 'displaying' stamp: ''!
text
	^ 'Object subclass: #NameOfSubclass
	instanceVariableNames: ''''
	classVariableNames: ''''
	poolDictionaries: ''''
	category: ''', self name, ''''! !

!OBCodeNode methodsFor: 'displaying' stamp: 'lr 5/2/2007 17:30'!
annotationString
	"Provide a line of content for an annotation pane, representing information about the method associated with the selected node."
	
	^ nil! !

!OBCodeNode methodsFor: 'testing' stamp: 'cwp 6/30/2006 00:37'!
hasOrganization
	^ false! !

!OBCodeNode methodsFor: 'testing' stamp: 'cwp 10/11/2006 20:47'!
hasSelector
	^ false! !

!OBCodeNode methodsFor: 'testing' stamp: 'cwp 10/9/2006 18:12'!
hasVersions
	^ false! !

!OBCodeNode methodsFor: 'testing' stamp: 'cwp 7/17/2006 08:46'!
isCategory
	^ false! !

!OBCodeNode methodsFor: 'testing' stamp: 'cwp 10/9/2006 15:05'!
isEnvironment
	^ false! !

OBCodeNode subclass: #OBEnvironmentNode
	instanceVariableNames: 'environment'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OB-Standard-Nodes'!

!OBEnvironmentNode commentStamp: 'cwp 1/8/2005 13:01' prior: 0!
OBEnvironmentNode wraps an instance of SystemDictionary. In current Squeak images, there is only one such instance, but OB-Standard is coded so as to use rely on this assumption as little as possible. Thus OBEnvironmentNode typically serves as the root of the standard browser, and passes its environment on to other nodes in the tree.!

!OBEnvironmentNode class methodsFor: 'as yet unclassified' stamp: ''!
forImage
	^ self on: Smalltalk! !

!OBEnvironmentNode class methodsFor: 'as yet unclassified' stamp: ''!
on: anEnvironment
	^ self new setEnvironment: anEnvironment! !

!OBEnvironmentNode methodsFor: 'navigating' stamp: ''!
categories
	^ environment organization categories collect: [:cat | OBClassCategoryNode 
															on: cat
															inEnvironment: environment]! !

!OBEnvironmentNode methodsFor: 'navigating' stamp: 'cwp 9/20/2005 08:43'!
categoryNodeNamed: aString 
	^ OBClassCategoryNode on: aString! !

!OBEnvironmentNode methodsFor: 'accessing' stamp: ''!
environment
	^ environment! !

!OBEnvironmentNode methodsFor: 'testing' stamp: 'cwp 6/30/2006 00:54'!
hasOrganization
	^ true! !

!OBEnvironmentNode methodsFor: 'testing' stamp: 'cwp 10/9/2006 13:36'!
isEnvironment
	^ true! !

!OBEnvironmentNode methodsFor: 'displaying' stamp: ''!
name 
	^ 'Squeak'! !

!OBEnvironmentNode methodsFor: 'accessing' stamp: ''!
organization
	^ self environment organization! !

!OBEnvironmentNode methodsFor: 'initializing' stamp: ''!
setEnvironment: anEnvironment
	environment := anEnvironment! !

OBCodeNode subclass: #OBSearchNode
	instanceVariableNames: 'query'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OB-Standard-Nodes'!

OBSearchNode subclass: #OBClassRefSearchNode
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OB-Standard-Nodes'!

!OBClassRefSearchNode methodsFor: 'as yet unclassified' stamp: 'cwp 4/15/2007 21:27'!
classSatisfiesQuery: aClass
	^ (SystemNavigation default allCallsOn: (Smalltalk associationAt: query) from: aClass) isEmpty not! !

!OBClassRefSearchNode methodsFor: 'as yet unclassified' stamp: 'cwp 4/14/2007 00:17'!
classes
	| refs classNames nodes |
	refs _ SystemNavigation default allCallsOn: (Smalltalk associationAt: query).
	classNames _ refs collect: [:ea | ea classSymbol].
	nodes _ classNames asSet collect: [:ea | OBClassNode on: (Smalltalk at: ea)].
	^ nodes asArray sort: [:a :b | a theClassName <= b theClassName]! !

!OBClassRefSearchNode methodsFor: 'as yet unclassified' stamp: 'cwp 4/7/2007 23:30'!
name
	^ query copyWithFirst: $#! !

OBSearchNode subclass: #OBClassSearchNode
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OB-Standard-Nodes'!

!OBClassSearchNode methodsFor: 'private' stamp: 'cwp 4/15/2007 20:44'!
classSatisfiesQuery: aClass
	^ aClass name includesSubstring: query caseSensitive: true! !

!OBClassSearchNode methodsFor: 'navigation' stamp: 'cwp 4/15/2007 20:52'!
classes
	^ (SystemNavigation default allClasses
		select: [:ea | self classSatisfiesQuery: ea]
		thenCollect: [:ea | OBClassNode on: ea]) asArray! !

!OBClassSearchNode methodsFor: 'as yet unclassified' stamp: 'cwp 4/19/2007 00:08'!
name
	^ query, '*'! !

!OBClassSearchNode methodsFor: 'accessing' stamp: 'cwp 4/3/2007 22:20'!
query
	^ query! !

OBSearchNode subclass: #OBImplementorSearchNode
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OB-Standard-Nodes'!

!OBImplementorSearchNode methodsFor: 'as yet unclassified' stamp: 'cwp 4/15/2007 20:40'!
classSatisfiesQuery: aClass
	^ aClass includesSelector: query! !

!OBImplementorSearchNode methodsFor: 'as yet unclassified' stamp: 'cwp 4/15/2007 21:43'!
classes
	
	^ (SystemNavigation default allClassesImplementing: query)
		collect: [:ea | OBClassNode on: ea]! !

OBSearchNode subclass: #OBMethodSearchNode
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OB-Standard-Nodes'!

!OBMethodSearchNode methodsFor: 'as yet unclassified' stamp: 'cwp 4/19/2007 00:39'!
classSatisfiesQuery: aClass
	^ aClass selectors anySatisfy: [:ea | ea includesSubstring: query caseSensitive: true]! !

!OBMethodSearchNode methodsFor: 'as yet unclassified' stamp: 'cwp 4/19/2007 20:58'!
classes
	| navigation classes |
	navigation _ self systemNavigation.
	classes _ IdentitySet new.
	(Symbol selectorsContaining: query) 
		do: [:ea | classes addAll: (navigation allClassesImplementing: ea)].
	^ classes asArray collect: [:ea | OBClassNode on: ea]! !

!OBMethodSearchNode methodsFor: 'as yet unclassified' stamp: 'cwp 4/19/2007 00:40'!
name
	^ query, '*'! !

!OBSearchNode class methodsFor: 'as yet unclassified' stamp: 'cwp 4/7/2007 23:28'!
on: aString
	^ self basicNew setQuery: aString! !

!OBSearchNode methodsFor: 'comparing' stamp: 'cwp 4/19/2007 00:36'!
= other
	^ self species = other species and: [self query = other query]! !

!OBSearchNode methodsFor: 'comparing' stamp: 'cwp 4/19/2007 00:36'!
hash
	^ query hash! !

!OBSearchNode methodsFor: 'ancestry' stamp: 'cwp 4/15/2007 20:31'!
isAncestorOf: aNode
	^ self = aNode or: [self classSatisfiesQuery: aNode theClass]! !

!OBSearchNode methodsFor: 'ancestry' stamp: 'cwp 5/12/2007 23:12'!
isAncestorOf: aNode using: aSelector
	^ self = aNode or: [self classSatisfiesQuery: aNode theClass]! !

!OBSearchNode methodsFor: 'display' stamp: 'cwp 4/7/2007 23:28'!
name
	^ query! !

!OBSearchNode methodsFor: 'private' stamp: 'cwp 4/15/2007 20:14'!
nodesForRefs: refs	
	| classNames nodes |
	classNames _ refs collect: [:ea | ea classSymbol].
	nodes _ classNames asSet collect: [:ea | OBClassNode on: (Smalltalk at: ea)].
	^ nodes asArray sort: [:a :b | a theClassName <= b theClassName]! !

!OBSearchNode methodsFor: 'accessing' stamp: 'cwp 4/8/2007 00:26'!
query
	^ query! !

!OBSearchNode methodsFor: 'initialize-release' stamp: 'cwp 4/7/2007 23:29'!
setQuery: aString
	query _ aString! !

OBSearchNode subclass: #OBSenderSearchNode
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OB-Standard-Nodes'!

!OBSenderSearchNode methodsFor: 'as yet unclassified' stamp: 'cwp 4/15/2007 21:27'!
classSatisfiesQuery: aClass
	^ (SystemNavigation default allCallsOn: query from: aClass) isEmpty not! !

!OBSenderSearchNode methodsFor: 'as yet unclassified' stamp: 'cwp 4/15/2007 21:32'!
classes
	^ self nodesForRefs: (SystemNavigation default allCallsOn: query)
	! !

!OBSenderSearchNode methodsFor: 'as yet unclassified' stamp: 'cwp 4/15/2007 21:17'!
name
	^ query copyWithFirst: $#! !

OBCodeNode subclass: #OBSelectorNode
	instanceVariableNames: 'selector methods'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OB-Standard-Nodes'!

!OBSelectorNode commentStamp: 'cwp 1/8/2005 13:03' prior: 0!
OBSelectorNode wraps an instance of Symbol. It typically serves as the root of a senders or implementors OBListBrowser, and is used when cmd-n or cmd-m is invoke from the definition panel, and so no OBMethodNode is available.!

!OBSelectorNode class methodsFor: 'instance creation' stamp: 'cwp 10/31/2004 01:11'!
on: aSelector
	^ self new selector: aSelector! !

!OBSelectorNode methodsFor: 'testing' stamp: 'cwp 10/11/2006 20:56'!
hasSelector
	^ true! !

!OBSelectorNode methodsFor: 'navigating' stamp: 'cwp 12/8/2004 23:48'!
implementors
	^ (SystemNavigation default allImplementorsOf: self selector) asSortedArray
			collect: [:ref | OBMethodNode on: ref]! !

!OBSelectorNode methodsFor: 'ancestry' stamp: 'DM 5/23/2007 18:20'!
isAncestorOf: aNode 
	^ (aNode source findString: selector string startingAt: 0 ) isNonZero.! !

!OBSelectorNode methodsFor: 'navigating' stamp: 'DM 5/23/2007 20:26'!
methods
	methods ifNil: 
			[methods := SystemNavigation default allMethodsNoDoitsSelect: 
							[:m | 
							m hasLiteralSuchThat: 
									[:lit | 
									(lit isString and: [lit isSymbol not]) 
										and: [lit includesSubstring: selector string caseSensitive: false]]]].
	^ methods collect: [:m | OBMethodNode on: m]! !

!OBSelectorNode methodsFor: 'navigating' stamp: 'cwp 10/31/2004 01:12'!
name
	^ selector printString! !

!OBSelectorNode methodsFor: 'accessing' stamp: 'cwp 10/31/2004 01:11'!
selector
	^ selector! !

!OBSelectorNode methodsFor: 'accessing' stamp: 'cwp 10/31/2004 01:10'!
selector: aSelector
	selector := aSelector! !

!OBSelectorNode methodsFor: 'navigating' stamp: 'cwp 10/31/2004 01:12'!
selectorAndMessages
	^ Array with: self! !

!OBSelectorNode methodsFor: 'navigating' stamp: 'cwp 11/1/2004 02:22'!
senders
	^ (SystemNavigation default allCallsOn: self selector) asSortedArray
			collect: [:ref | OBMessageNode on: self selector inMethodReference: ref]! !

!OBSelectorNode methodsFor: 'navigating' stamp: 'cwp 10/31/2004 01:48'!
sendersOfMessage
	^ (SystemNavigation default allCallsOn: self selector) asSortedArray
			collect: [:ref | OBMessageNode on: self selector inMethodReference: ref]! !

!OBCollectionNode methodsFor: '*ob-standard' stamp: 'cwp 4/1/2007 22:59'!
hasSelector
	^ false! !

!OBCollectionNode methodsFor: '*ob-standard' stamp: 'cwp 4/1/2007 23:00'!
hasVersions
	^ false! !

!Metaclass methodsFor: '*ob-standard-converting' stamp: 'cwp 10/23/2005 14:40'!
asClassSideNode
	^ OBMetaclassNode on: self theNonMetaClass! !

!Metaclass methodsFor: '*ob-standard-converting' stamp: ''!
asNode
	^ (OBMetaclassNode on: self)! !

OBBrowser subclass: #OBCodeBrowser
	instanceVariableNames: 'hasChanges'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OB-Standard-Browsers'!

!OBCodeBrowser commentStamp: 'cwp 1/7/2005 23:45' prior: 0!
OBCodeBrowser is a superclass for all browsers which active code in the image. It provides methods for registering with the SystemChangeNotifier and updating it's display when it receives notification of system changes.!

!OBCodeBrowser class methodsFor: 'configuration' stamp: 'lr 5/2/2007 17:15'!
annotationPanel
	^ Preferences annotationPanes ifTrue: [ OBAnnotationPanel new ]! !

!OBCodeBrowser class methodsFor: 'configuration' stamp: 'cwp 5/20/2007 00:02'!
buildMetagraphOn: root 
	^ self 
		buildMetagraphOn: root
		class: #classes
		comment: #comments
		metaclass: #metaclasses! !

!OBCodeBrowser class methodsFor: 'configuration' stamp: 'cwp 5/20/2007 02:42'!
buildMetagraphOn: root class: classSel comment: commentSel metaclass: metaclassSel
	| class metaclass comment methodCategory method allMethodCategory |
	class := OBMetaNode named: 'Class'.
	comment := OBMetaNode named: 'ClassComment'.
	metaclass := OBMetaNode named: 'Metaclass'.
	allMethodCategory := OBMetaNode named: 'AllMethodCategory'.
	methodCategory := OBMetaNode named: 'MethodCategory'.
	method := OBMetaNode named: 'Method'.
	root 
		childAt: classSel labeled: 'instance' put: class;
		childAt: commentSel labeled: '?' put: comment;
		childAt: metaclassSel labeled: 'class' put: metaclass;
		addFilter: OBModalFilter new;
		addFilter: OBClassSortFilter new.
	class
		ancestrySelector: #isDescendantOfClass:;
		childAt: #allCategory put: allMethodCategory;
		childAt: #categories put: methodCategory.
	metaclass
		ancestrySelector: #isDescendantOfClass:;
		childAt: #allCategory put: allMethodCategory;
		childAt: #categories put: methodCategory.
	allMethodCategory childAt: #methods put: method.
	methodCategory
		ancestrySelector: #isDescendantOfMethodCat:;
		childAt: #methods put: method.
	method ancestrySelector: #isDescendantOfMethod:.
	^root! !

!OBCodeBrowser class methodsFor: 'configuration' stamp: 'avi 5/30/2007 22:09'!
chasingBrowsers
	^ Preferences chasingBrowsers! !

!OBCodeBrowser class methodsFor: 'configuration' stamp: 'avi 5/30/2007 22:10'!
initialize
	"OBCodeBrowser initialize"
	Preferences 
		addPreference: #chasingBrowsers
		categories: #('browsing')
		default: true
		balloonHelp: 'Use recursive multi-column browsers for senders, implementors, etc.'! !

!OBCodeBrowser class methodsFor: 'configuration' stamp: 'ms 5/8/2007 18:33'!
mercuryPanel
	^ Preferences mercuryPanel ifTrue:[OBMercuryPanel new] ifFalse:[nil]! !

!OBCodeBrowser class methodsFor: 'configuration' stamp: 'cwp 12/15/2004 22:52'!
optionalButtons
	^ #('browse' 'variables' 'hierarchy' 'inheritance' 'senders' 'implementors' 'versions')! !

!OBCodeBrowser class methodsFor: 'configuration' stamp: 'lr 5/2/2007 17:13'!
panels
	^ {self mercuryPanel. self navigationPanel. self annotationPanel. self optionalButtonPanel. self definitionPanel} reject: [:ea | ea isNil]! !

!OBCodeBrowser methodsFor: 'morphic' stamp: 'cwp 1/8/2005 21:55'!
addModelItemsToWindowMenu: aMenu
	Smalltalk 
		at: #SystemBrowser 
		ifPresent: [:class | class 
								addRegistryMenuItemsTo: aMenu 
								inAccountOf: OBSystemBrowserAdaptor new].! !

!OBCodeBrowser methodsFor: 'updating' stamp: 'cwp 1/7/2005 23:47'!
clearChanges
	hasChanges := false! !

!OBCodeBrowser methodsFor: 'commands' stamp: 'cwp 11/1/2006 23:17'!
cmdAutoCategorizeCat
	^OBCmdAutoCategorize! !

!OBCodeBrowser methodsFor: 'commands' stamp: 'cwp 11/1/2006 23:17'!
cmdBrowse
	^OBCmdBrowse! !

!OBCodeBrowser methodsFor: 'commands' stamp: 'cwp 11/1/2006 23:18'!
cmdBrowseHierarchy
	^OBCmdBrowseHierarchy! !

!OBCodeBrowser methodsFor: 'commands' stamp: 'cwp 11/1/2006 23:18'!
cmdBrowseImplementors
	^OBCmdBrowseImplementors! !

!OBCodeBrowser methodsFor: 'commands' stamp: 'cwp 11/1/2006 23:18'!
cmdBrowseImplementorsOfIt
	^OBCmdBrowseImplementorsOfIt! !

!OBCodeBrowser methodsFor: 'commands' stamp: 'cwp 11/1/2006 23:18'!
cmdBrowseInheritance
	^OBCmdBrowseInheritance! !

!OBCodeBrowser methodsFor: 'commands' stamp: 'cwp 11/1/2006 23:19'!
cmdBrowseMethodVersions
	^OBCmdBrowseMethodVersions! !

!OBCodeBrowser methodsFor: 'commands' stamp: 'jk 5/18/2007 10:08'!
cmdBrowseProtocol
	^ OBCmdBrowseProtocol ! !

!OBCodeBrowser methodsFor: 'commands' stamp: 'cwp 12/7/2006 22:17'!
cmdBrowseReferences
	^ OBCmdBrowseReferences! !

!OBCodeBrowser methodsFor: 'commands' stamp: 'cwp 11/1/2006 23:19'!
cmdBrowseSenders
	^OBCmdBrowseSenders! !

!OBCodeBrowser methodsFor: 'commands' stamp: 'cwp 11/1/2006 23:37'!
cmdBrowseSendersOfIt
	^OBCmdBrowseSendersOfIt! !

!OBCodeBrowser methodsFor: 'commands' stamp: 'cwp 11/1/2006 23:37'!
cmdChaseVariables
	^OBCmdChaseVariables! !

!OBCodeBrowser methodsFor: 'commands' stamp: 'cwp 11/2/2006 21:46'!
cmdCopyClass
	^ OBCmdCopyClass! !

!OBCodeBrowser methodsFor: 'commands' stamp: 'cwp 11/1/2006 23:37'!
cmdCreateCat
	^OBCmdCreateCategory! !

!OBCodeBrowser methodsFor: 'commands' stamp: 'cwp 11/1/2006 23:37'!
cmdFileOut
	^OBCmdFileOut! !

!OBCodeBrowser methodsFor: 'commands' stamp: 'cwp 11/1/2006 23:38'!
cmdFindClass
	^OBCmdFindClass! !

!OBCodeBrowser methodsFor: 'commands' stamp: 'cwp 11/1/2006 23:38'!
cmdFindMethod
	^OBCmdFindMethod! !

!OBCodeBrowser methodsFor: 'commands' stamp: 'DM 5/23/2007 20:37'!
cmdMethodsContainingIt
	^ OBCmdBrowseMethodsWithIt! !

!OBCodeBrowser methodsFor: 'commands' stamp: 'cwp 12/11/2006 21:27'!
cmdMoveToPackage
	^ OBCmdMoveToPackage! !

!OBCodeBrowser methodsFor: 'commands' stamp: 'cwp 11/1/2006 23:38'!
cmdRemoveCat
	^OBCmdRemoveCat! !

!OBCodeBrowser methodsFor: 'commands' stamp: 'cwp 11/3/2006 00:14'!
cmdRemoveClass
	^ OBCmdRemoveClass! !

!OBCodeBrowser methodsFor: 'commands' stamp: 'cwp 11/1/2006 23:38'!
cmdRemoveEmptyCats
	^OBCmdRemoveEmptyCats! !

!OBCodeBrowser methodsFor: 'commands' stamp: 'cwp 12/9/2006 21:43'!
cmdRemoveMethod
	^ OBCmdRemoveMethod! !

!OBCodeBrowser methodsFor: 'commands' stamp: 'cwp 11/1/2006 23:39'!
cmdRenameCat
	^OBCmdRenameCat! !

!OBCodeBrowser methodsFor: 'commands' stamp: 'cwp 11/2/2006 23:57'!
cmdRenameClass
	^ OBCmdRenameClass! !

!OBCodeBrowser methodsFor: 'commands' stamp: 'cwp 11/1/2006 23:39'!
cmdReorgCats
	^OBCmdReorgCats! !

!OBCodeBrowser methodsFor: 'commands' stamp: 'cwp 11/1/2006 23:39'!
cmdRevertMethod
	^OBCmdRevertMethod! !

!OBCodeBrowser methodsFor: 'commands' stamp: 'cwp 11/1/2006 23:40'!
cmdSortCats
	^OBCmdSortCats! !

!OBCodeBrowser methodsFor: 'commands' stamp: 'cwp 12/7/2006 23:40'!
cmdSubclassTemplate
	^ OBCmdSubclassTemplate! !

!OBCodeBrowser methodsFor: 'updating' stamp: 'cwp 4/15/2007 23:08'!
event: anEvent
	self noteChanges! !

!OBCodeBrowser methodsFor: 'morphic' stamp: ''!
initialExtent
	hasChanges := false.
	self register.
	^ super initialExtent! !

!OBCodeBrowser methodsFor: 'testing' stamp: 'cwp 4/18/2007 23:48'!
isClassBrowser
	^ false! !

!OBCodeBrowser methodsFor: 'testing' stamp: 'cwp 4/18/2007 23:48'!
isSearchBrowser
	^ false! !

!OBCodeBrowser methodsFor: 'updating' stamp: 'cwp 4/15/2007 23:07'!
noteChanges
	hasChanges := true! !

!OBCodeBrowser methodsFor: 'updating' stamp: 'cwp 3/1/2006 19:48'!
register
	SystemChangeNotifier uniqueInstance notify: self ofAllSystemChangesUsing: #event:! !

!OBCodeBrowser methodsFor: 'morphic' stamp: 'cwp 6/4/2006 18:51'!
stepAt: milliseconds in: aSystemWindow
	hasChanges ifTrue: [self signalRefresh].
	self clearChanges! !

!OBCodeBrowser methodsFor: 'updating' stamp: 'cwp 3/1/2006 19:48'!
unregister
	SystemChangeNotifier uniqueInstance noMoreNotificationsFor: self! !

!OBCodeBrowser methodsFor: 'morphic' stamp: ''!
wantsStepsIn: aSystemWindow
	^ true! !

!OBCodeBrowser methodsFor: 'morphic' stamp: ''!
windowIsClosing
	self unregister! !

OBCodeBrowser subclass: #OBHierarchyBrowser
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OB-Standard-Browsers'!

!OBHierarchyBrowser commentStamp: 'cwp 1/7/2005 23:50' prior: 0!
OBHierarchyBrowser provides a three-pane browers that displays a class within it's surrounding hierarchy - both superclasses and subclasses!

!OBHierarchyBrowser class methodsFor: 'configuration' stamp: 'cwp 5/20/2007 00:05'!
defaultMetaNode
	^ self 
		buildMetagraphOn: (OBMetaNode named: 'RootClass')
		class: #classHierarchy 
		comment: #commentHierarchy 
		metaclass: #metaclassHierarchy.
! !

!OBHierarchyBrowser class methodsFor: 'instance creation' stamp: 'lr 3/6/2006 19:28'!
onClass: aClass 
	^self root: aClass asNode selection: aClass asNode! !

!OBHierarchyBrowser class methodsFor: 'opening' stamp: 'cwp 12/15/2004 22:49'!
openOnClass: aClass
	^ (self onClass: aClass) open! !

!OBHierarchyBrowser class methodsFor: 'configuration' stamp: 'cwp 12/13/2004 00:37'!
paneCount
	^ 3! !

!OBHierarchyBrowser class methodsFor: 'configuration' stamp: 'cwp 12/13/2004 01:00'!
titleForRoot: aNode
	^ aNode name, ' Hierarchy'! !

!OBHierarchyBrowser methodsFor: 'as yet unclassified' stamp: 'cwp 12/13/2004 00:40'!
defaultBackgroundColor
	^ Color lightGreen! !

!OBHierarchyBrowser methodsFor: 'as yet unclassified' stamp: 'cwp 12/13/2004 01:00'!
defaultLabel
	^ self root name, ' Hierarchy'! !

OBCodeBrowser subclass: #OBInheritanceBrowser
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OB-Standard-Browsers'!

!OBInheritanceBrowser commentStamp: 'cwp 1/7/2005 23:51' prior: 0!
OBInheritanceBrowser shows the inheritance hierarchy of a method, both superclass implementations which it overrides, and subclass implementations which override it.!

!OBInheritanceBrowser class methodsFor: 'configuration' stamp: 'cwp 5/14/2007 13:45'!
defaultMetaNode
	| method root |
	method := OBMetaNode named: 'Method'.
	root := OBMetaNode named: 'Root'.
	root
		childAt: #children put: method.
	method
		displaySelector: #fullName;
		ancestrySelector: #isDescendantOfMethod:;
		childAt: #overrides put: method.
	^ root! !

!OBInheritanceBrowser class methodsFor: 'configuration' stamp: ''!
title
	^ 'Inheritance'! !

!OBInheritanceBrowser class methodsFor: 'configuration' stamp: 'cwp 11/25/2004 22:05'!
titleForRoot: aCollectionNode
	^ 'Inheritance of ', aCollectionNode children first selector printString! !

!OBInheritanceBrowser methodsFor: 'morphic' stamp: 'avi 11/29/2004 21:52'!
defaultBackgroundColor
	^ Color lightGreen! !

OBCodeBrowser subclass: #OBListBrowser
	instanceVariableNames: 'labelPrefix'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OB-Standard-Browsers'!

!OBListBrowser commentStamp: 'cwp 1/7/2005 23:53' prior: 0!
OBListBrowsers are used to display simple lists of methods, such as senders or implementors.!

OBListBrowser subclass: #OBImplementorsBrowser
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OB-Standard-Browsers'!

!OBImplementorsBrowser class methodsFor: 'as yet unclassified' stamp: 'avi 5/30/2007 22:11'!
defaultMetaNode
	| selector message implementor |
	selector := OBMetaNode named: 'Selector'.
	message := OBMetaNode named: 'Message'.
	implementor := OBMetaNode named: 'Implementor'.

	selector 
		childAt: #implementors put: implementor;
		addFilter: OBClassSortFilter new.
	implementor
		displaySelector: #fullName.
	self chasingBrowsers ifTrue:
		[implementor
			childAt: #messages put: message.
		message childAt: #implementors put: implementor].
	^selector! !

!OBImplementorsBrowser class methodsFor: 'as yet unclassified' stamp: 'avi 9/17/2005 01:36'!
title
	^ 'Implementors of'! !

!OBListBrowser class methodsFor: 'opening' stamp: 'avi 9/17/2005 01:34'!
browseRoot: aNode
	self browseRoot: aNode title: self title! !

!OBListBrowser class methodsFor: 'opening' stamp: 'avi 9/17/2005 01:21'!
browseRoot: aNode title: aString 
	(self metaNode: self defaultMetaNode root: aNode selection: nil)
		labelPrefix: aString;
		open
	! !

!OBListBrowser class methodsFor: 'configuration' stamp: 'cwp 12/5/2004 17:54'!
maxPanes
	^ 2! !

!OBListBrowser class methodsFor: 'configuration' stamp: 'cwp 12/5/2004 17:54'!
minPanes
	^ 1! !

!OBListBrowser methodsFor: 'morphic' stamp: 'avi 11/29/2004 21:58'!
defaultBackgroundColor
	^ Color lightBlue! !

!OBListBrowser methodsFor: 'accessing' stamp: 'cwp 11/25/2004 22:21'!
defaultLabel
	^ self labelPrefix, ' ', self root name printString! !

!OBListBrowser methodsFor: 'accessing' stamp: 'cwp 11/25/2004 22:18'!
labelPrefix
	^ labelPrefix ifNil: [labelPrefix _ self root metaNode edges first label capitalized, ' of']! !

!OBListBrowser methodsFor: 'accessing' stamp: 'cwp 10/17/2004 20:54'!
labelPrefix: aString
	labelPrefix _ aString! !

OBListBrowser subclass: #OBMethodStringsBrowser
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OB-Standard-Browsers'!

!OBMethodStringsBrowser class methodsFor: 'as yet unclassified' stamp: 'DM 5/23/2007 20:21'!
browseRoot: aNode target: targetNode
	(self root: aNode selection: targetNode) open.! !

!OBMethodStringsBrowser class methodsFor: 'as yet unclassified' stamp: 'DM 5/23/2007 20:43'!
defaultMetaNode
	| matches selection |
	selection := OBMetaNode named: 'Selection'.
	matches := OBMetaNode named: 'Methods'.
	selection childAt: #methods put: matches.
	matches
		displaySelector: #fullName;
		ancestrySelector: #isAncestorOf:.
	^ selection! !

!OBMethodStringsBrowser methodsFor: 'as yet unclassified' stamp: 'DM 5/24/2007 20:12'!
defaultBackgroundColor
	^Preferences messageListWindowColor! !

!OBMethodStringsBrowser methodsFor: 'as yet unclassified' stamp: 'DM 5/23/2007 20:24'!
defaultLabel
	^ 'Methods with literal', self selectorString, self numOfMatchesString! !

!OBMethodStringsBrowser methodsFor: 'as yet unclassified' stamp: 'DM 5/23/2007 20:26'!
numOfMatchesString
	^ '[' , self root methods size asString , ']'! !

!OBMethodStringsBrowser methodsFor: 'as yet unclassified' stamp: 'DM 5/23/2007 20:23'!
selectorString
	^ ' ''', self root selector string, ''' '! !

OBListBrowser subclass: #OBReferencesBrowser
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OB-Standard-Browsers'!

!OBReferencesBrowser class methodsFor: 'configuration' stamp: 'avi 5/30/2007 22:12'!
defaultMetaNode
	| class method sender |
	class := OBMetaNode named: 'Class'.
	method := OBMetaNode named: 'References'.
	sender := OBMetaNode named: 'Sender'.
	
	class childAt: #users put: method.
	method 
		displaySelector: #fullName.
	self chasingBrowsers ifTrue:
		[method
			childAt: #senders put: sender.
		sender
			displaySelector: #fullName;
			childAt: #senders put: sender].

	^ class! !

!OBReferencesBrowser class methodsFor: 'configuration' stamp: 'avi 9/17/2005 01:34'!
title
	^ 'References to'! !

OBListBrowser subclass: #OBSendersBrowser
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OB-Standard-Browsers'!

!OBSendersBrowser class methodsFor: 'as yet unclassified' stamp: 'avi 5/30/2007 22:12'!
defaultMetaNode
	| selector sender |
	selector := OBMetaNode named: 'Selector'.	
	sender := OBMetaNode named: 'Sender'.
	selector childAt: #senders put: sender.
	sender
		displaySelector: #fullName.
	self chasingBrowsers ifTrue:
		[sender
			childAt: #senders put: sender].
	^ selector! !

!OBSendersBrowser class methodsFor: 'as yet unclassified' stamp: 'avi 9/17/2005 01:36'!
title
	^ 'Senders of'! !

OBListBrowser subclass: #OBVariablesBrowser
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OB-Standard-Browsers'!

!OBVariablesBrowser class methodsFor: 'as yet unclassified' stamp: 'avi 5/30/2007 22:25'!
defaultMetaNode
	| class method message var |
	class := OBMetaNode named: 'Class'.
	var := OBMetaNode named: 'Variable'.
	method := OBMetaNode named: 'Method'.
	message := OBMetaNode named: 'Message'.
	class
		childAt: #instanceVariables labeled: 'instance' put: var;
		childAt: #classVariables labeled: 'class' put: var;
		addFilter: OBModalFilter new.
	var
		childAt: #accessors put: method;
		addFilter: OBClassSortFilter new.
	method
		displaySelector: #fullName.
	self chasingBrowsers ifTrue:
		[method
			childAt: #senders put: message;
			addFilter: OBModalFilter new.
		var
			addFilter: OBModalFilter new.
		message
			displaySelector: #fullName;
			childAt: #senders put: message;
			addFilter: OBModalFilter new].
	
	^ class! !

!OBVariablesBrowser class methodsFor: 'as yet unclassified' stamp: 'avi 5/30/2007 22:13'!
minPanes
	^ 2! !

!OBVariablesBrowser class methodsFor: 'as yet unclassified' stamp: 'avi 9/17/2005 01:35'!
title
	^ 'Variables of'! !

OBCodeBrowser subclass: #OBProtocolBrowser
	instanceVariableNames: 'theClass limitClass filter'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OB-Standard-Browsers'!

!OBProtocolBrowser class methodsFor: 'configuration' stamp: 'DM 5/23/2007 15:33'!
defaultMetaNode
	| protocol method class |
	class := OBMetaNode named: 'Class'.
	protocol := OBMetaNode named: 'Protocol'.
	method := OBMetaNode named: 'Method'.
	class 
		ancestrySelector: #isDescendantOfClass:;
		childAt: #allCategories put: protocol.
	protocol 
		ancestrySelector: #isDescendantOfMethodCat:;
		childAt: #allMethods put: method.
	method 
		ancestrySelector: #isDescendantOfMethod:;
		displaySelector: #nameWithClassNameInBrackets.
	^ class! !

!OBProtocolBrowser class methodsFor: 'configuration' stamp: 'DM 5/20/2007 15:31'!
on: aNode 
	^ (self root: aNode selection: aNode) open! !

!OBProtocolBrowser class methodsFor: 'configuration' stamp: 'DM 5/20/2007 02:32'!
paneCount
^ 2.! !

!OBProtocolBrowser methodsFor: 'commands' stamp: 'DM 5/21/2007 16:36'!
cmdFilterProtocol
	^ OBCmdLimitProtocol! !

!OBProtocolBrowser methodsFor: 'building' stamp: 'cwp 5/31/2007 21:26'!
defaultBackgroundColor
	| color |
	color _ Preferences lexiconWindowColor.
	^ color == false 
		ifTrue: [Color lightGreen]
		ifFalse: [color]! !

!OBProtocolBrowser methodsFor: 'accessing' stamp: 'DM 5/20/2007 15:33'!
defaultLabel
	^ 'Vocabulary of ' , self root theClass name.! !

!OBProtocolBrowser methodsFor: 'accessing' stamp: 'DM 5/22/2007 18:41'!
defaultLimitClass
	^ (theClass isKindOf: Metaclass) 
		ifTrue: [Object class]
		ifFalse: [Object]! !

!OBProtocolBrowser methodsFor: 'accessing' stamp: 'DM 5/22/2007 18:42'!
limitClass
	^ limitClass 
		ifNil: [self defaultLimitClass]
		ifNotNil: [limitClass]! !

!OBProtocolBrowser methodsFor: 'accessing' stamp: 'DM 5/31/2007 14:55'!
limitClass: ann
	limitClass := ann node.
	filter limitClass: ann node! !

!OBProtocolBrowser methodsFor: 'updating' stamp: 'DM 5/31/2007 14:54'!
refresh: ann 
	panels do: [:p | p changed: #commands].! !

!OBProtocolBrowser methodsFor: 'initialization' stamp: 'DM 5/23/2007 14:38'!
setMetaNode: aMetaNode node: aNode 
	super setMetaNode: aMetaNode node: aNode.
	theClass := aNode theClass.
	filter := OBLimitProtocolFilter on: aNode theClass.
	aMetaNode addFilter: filter! !

!OBProtocolBrowser methodsFor: 'updating' stamp: 'DM 5/31/2007 14:57'!
subscribe
	super subscribe.
	self announcer
		observe: OBRefreshRequired  
		send:  #refresh:
		to: self;
		observe: OBNodeChanged 
		send:  #limitClass: 
		to: self.! !

!OBProtocolBrowser methodsFor: 'private' stamp: 'DM 5/23/2007 15:25'!
theClass
	^ theClass.! !

OBCodeBrowser subclass: #OBSearchBrowser
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OB-Standard-Browsers'!

!OBSearchBrowser class methodsFor: 'as yet unclassified' stamp: 'cwp 5/19/2007 23:33'!
defaultMetaNode
	| root search |
	root := OBMetaNode named: 'root'.
	search := OBMetaNode named: 'search'.
	root childAt: #children put: search.
	self buildMetagraphOn: search.
	^root! !

!OBSearchBrowser class methodsFor: 'as yet unclassified' stamp: 'cwp 4/1/2007 22:56'!
defaultRootNode
	^ OBCollectionNode on: IdentitySet new! !

!OBSearchBrowser class methodsFor: 'as yet unclassified' stamp: 'cwp 4/16/2007 00:17'!
initialize
	self registerInOpenMenu! !

!OBSearchBrowser class methodsFor: 'as yet unclassified' stamp: 'cwp 4/16/2007 00:16'!
registerInOpenMenu
	TheWorldMenu registerOpenCommand: {'Search Browser'. {self. #open}}.
! !

!OBSearchBrowser class methodsFor: 'as yet unclassified' stamp: 'cwp 4/16/2007 00:18'!
title
	^ 'Search Browser'! !

!OBSearchBrowser class methodsFor: 'as yet unclassified' stamp: 'cwp 4/15/2007 23:20'!
with: aSearchNode
	^ self
		metaNode: self defaultMetaNode
		root: (OBCollectionNode on: (OrderedCollection with: aSearchNode))
		selection: aSearchNode! !

!OBSearchBrowser methodsFor: 'as yet unclassified' stamp: 'cwp 4/1/2007 23:01'!
defaultBackgroundColor
	^ Color lightGreen

! !

!OBSearchBrowser methodsFor: 'as yet unclassified' stamp: 'cwp 4/15/2007 22:28'!
isSearchBrowser
	^ true! !

OBCodeBrowser subclass: #OBSystemBrowser
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OB-Standard-Browsers'!

!OBSystemBrowser commentStamp: 'cwp 1/7/2005 23:56' prior: 0!
This is the basic system browser that is the work-horse of Smalltalk development tools. It presents four panes showing class categories, classes, method categories and methods.!

!OBSystemBrowser class methodsFor: 'configuration' stamp: 'cwp 5/19/2007 23:36'!
defaultMetaNode
	| env classCategory |
	env := OBMetaNode named: 'Environment'.
	classCategory := OBMetaNode named: 'ClassCategory'.
	env childAt: #categories put: classCategory.
	classCategory ancestrySelector: #isDescendantOfClassCat:.
	self buildMetagraphOn: classCategory.
	^env! !

!OBSystemBrowser class methodsFor: 'configuration' stamp: 'cwp 12/6/2004 22:02'!
defaultRootNode
	^ OBEnvironmentNode forImage! !

!OBSystemBrowser class methodsFor: 'initializing' stamp: ''!
initialize
	self registerInOpenMenu! !

!OBSystemBrowser class methodsFor: 'instance creation' stamp: 'dvf 8/31/2005 13:17'!
onClass: aClass 
	^self selection: aClass asNode! !

!OBSystemBrowser class methodsFor: 'instance creation' stamp: 'cwp 12/15/2004 21:42'!
onClass: aClass category: aSymbol
	^ self selection: (OBMethodCategoryNode on: aSymbol inClass: aClass)! !

!OBSystemBrowser class methodsFor: 'instance creation' stamp: 'cwp 12/5/2004 17:38'!
onClass: aClass selector: aSelector
	^ self selection: (OBMethodNode on: aSelector inClass: aClass)! !

!OBSystemBrowser class methodsFor: 'instance creation' stamp: 'cwp 12/15/2004 21:45'!
onEnvironment: anEnvironment category: aSymbol
	^ self selection: (OBClassCategoryNode 
						on: aSymbol 
						inEnvironment: anEnvironment)! !

!OBSystemBrowser class methodsFor: 'opening' stamp: 'cwp 12/5/2004 18:32'!
openOnClass: aClass
	^ (self onClass: aClass) open! !

!OBSystemBrowser class methodsFor: 'opening' stamp: 'cwp 12/15/2004 21:47'!
openOnClass: aClass category: aSymbol
	^ (self onClass: aClass category: aSymbol) open! !

!OBSystemBrowser class methodsFor: 'opening' stamp: 'cwp 12/5/2004 17:38'!
openOnClass: aClass selector: aSelector
	^ (self onClass: aClass selector: aSelector) open! !

!OBSystemBrowser class methodsFor: 'opening' stamp: 'cwp 12/15/2004 21:54'!
openOnEnvironment: anEnvironment category: aSymbol
	^ (self onEnvironment: anEnvironment category: aSymbol) open! !

!OBSystemBrowser class methodsFor: 'initializing' stamp: ''!
registerInOpenMenu
	TheWorldMenu registerOpenCommand: {'Image Browser'. {self. #open}}.
! !

!OBSystemBrowser class methodsFor: 'configuration' stamp: ''!
title
	^ 'System Browser'! !

!OBSystemBrowser methodsFor: 'morphic' stamp: 'avi 11/29/2004 21:52'!
defaultBackgroundColor
	^ Color lightGreen

! !

!OBSystemBrowser methodsFor: 'testing' stamp: 'cwp 4/18/2007 23:49'!
isClassBrowser
	^ true! !

OBCodeBrowser subclass: #OBVersionBrowser
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OB-Standard-Browsers'!

!OBVersionBrowser commentStamp: 'cwp 1/7/2005 23:58' prior: 0!
OBVersionBrowser displays a list of OBMethodVersions, which represent versions of a method present in the source or changes files. !

!OBVersionBrowser class methodsFor: 'configuration' stamp: 'cwp 5/11/2007 01:17'!
defaultMetaNode
	| version |
	version := OBMetaNode named: 'Version'.
	version ancestrySelector: #isDescendantOfMethodVersion:.
	^ (OBMetaNode named: 'Method') 
		childAt: #versions put: version.! !

!OBVersionBrowser class methodsFor: 'configuration' stamp: 'cwp 11/27/2004 22:32'!
maxPanes
	^ 1! !

!OBVersionBrowser class methodsFor: 'configuration' stamp: 'cwp 11/27/2004 22:32'!
minPanes
	^ 1! !

!OBVersionBrowser class methodsFor: 'opening' stamp: 'lr 4/25/2007 09:51'!
openOn: aMethodVersionNode
	^ (self root: aMethodVersionNode selection: aMethodVersionNode versions first) open! !

!OBVersionBrowser class methodsFor: 'configuration' stamp: 'cwp 11/27/2004 22:34'!
optionalButtons
	^ #('browse' 'revert')! !

!OBVersionBrowser methodsFor: 'morphic' stamp: 'avi 11/29/2004 21:52'!
defaultBackgroundColor
	^ Color lightMagenta! !

!OBVersionBrowser methodsFor: 'accessing' stamp: 'cwp 11/27/2004 22:28'!
defaultLabel
	^ 'Versions of ', self root name printString! !

!Class methodsFor: '*ob-standard-converting' stamp: 'dvf 8/15/2005 17:52'!
asClassSideNode
	^OBMetaclassNode on: self! !

!Class methodsFor: '*ob-standard-converting' stamp: 'dvf 9/1/2005 13:51'!
asNode
	^OBClassNode on: self! !

!MethodReference methodsFor: '*OB-Standard' stamp: 'dvf 8/19/2005 17:16'!
asNode
	^OBMethodNode on: self! !

OBDefinition subclass: #OBClassDefinition
	instanceVariableNames: 'environment template'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OB-Standard-Definitions'!

!OBClassDefinition commentStamp: 'cwp 1/8/2005 13:13' prior: 0!
OBClassDefinition presents a textual interface for examining, modifying and creating classes. Given a class, it knows how to display the definition expression that reflects it's current state, and knows how to create or modify a class given a definition expression.

OBClassDefinition implements a number of safety checks when defining or redefining classes, to ensure that existing classes are not accidentally overwritten.!

!OBClassDefinition class methodsFor: 'instance creation' stamp: ''!
environment: anEnvironment template: aString 
	^ self new setEnvironment: anEnvironment template: aString! !

!OBClassDefinition methodsFor: 'callbacks' stamp: ''!
accept: aText notifying: aController
	^ self defineClass: aText notifying: aController! !

!OBClassDefinition methodsFor: 'confirmation' stamp: ''!
confirmDefinition: definition
	"Check to make sure the user isn't accidentally over-writing an existing class."
	
	(((self isRedefinition: definition) not) and: [self definedClassExists: definition])
		ifTrue: [^ self confirmRedefinition: definition]
		ifFalse: [^ true]! !

!OBClassDefinition methodsFor: 'confirmation' stamp: ''!
confirmRedefinition: definition
	| newName prompt |
	newName := self nameOfClassDefinedBy: definition.
	prompt := (newName, ' is an existing class in this system.
Redefining it might cause serious problems.
Is this really what you want to do?') asText.
	^ OBConfirmationRequest
		prompt: prompt
		confirm: 'Redefine'! !

!OBClassDefinition methodsFor: 'class definition' stamp: ''!
defineClass: definition notifying: aController
	| evaluator newClass |
	(self confirmDefinition: definition) ifFalse: [^ false].
	evaluator := self evaluatorForDefinition: definition.
	newClass := evaluator
				evaluate: definition
				notifying: aController
				logged: true.
	newClass ifNil: [^ false].
	self signalSelectionOf: newClass.
	^ true
! !

!OBClassDefinition methodsFor: 'confirmation' stamp: ''!
definedClassExists: definition
	^ environment hasClassNamed: (self nameOfClassDefinedBy: definition)! !

!OBClassDefinition methodsFor: 'class definition' stamp: ''!
evaluatorForDefinition: definition
	| tokens |
	tokens := Scanner new scanTokens: definition.
	^ (environment at: tokens first ifAbsent: [nil]) subclassDefinerClass! !

!OBClassDefinition methodsFor: 'confirmation' stamp: ''!
isRedefinition: aDefinition
	^ (self nameOfClassDefinedBy: aDefinition) = (self nameOfClassDefinedBy: template)! !

!OBClassDefinition methodsFor: 'confirmation' stamp: ''!
nameOfClassDefinedBy: definition
	^ (Scanner new scanTokens: definition) third! !

!OBClassDefinition methodsFor: 'confirmation' stamp: 'cwp 10/18/2004 00:13'!
selectedClass
	^ environment at: (self nameOfClassDefinedBy: template) ifAbsent: [nil]! !

!OBClassDefinition methodsFor: 'callbacks' stamp: ''!
selection
	^ 1 to: 0! !

!OBClassDefinition methodsFor: 'initializing' stamp: ''!
setEnvironment: anEnvironment template: aText
	environment := anEnvironment.
	template := aText.! !

!OBClassDefinition methodsFor: 'class definition' stamp: 'cwp 10/23/2005 14:42'!
signalSelectionOf: aClass
	aClass asNode demandSelection! !

!OBClassDefinition methodsFor: 'callbacks' stamp: ''!
text
	^ template! !

OBDefinition subclass: #OBMethodDefinition
	instanceVariableNames: 'theClass category source selection callback'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OB-Standard-Definitions'!

!OBMethodDefinition commentStamp: 'cwp 1/8/2005 13:20' prior: 0!
OBMethodDefinition knows how to present method source in the browser's text pane and compile a new CompiledMethod when the source changes. It implements several safety checks to ensure that methods are not accidentally overwritten.!

!OBMethodDefinition class methodsFor: 'instance creation' stamp: 'cwp 10/17/2004 23:07'!
inCategory: aString inClass: aClass
	^ self 
		selection: nil
		source: nil
		inCategory: aString
		inClass: aClass! !

!OBMethodDefinition class methodsFor: 'instance creation' stamp: 'cwp 10/17/2004 23:07'!
selection: anInterval source: aText inCategory: aCategory inClass: aClass
	^ self new
		setClass: aClass
		category: aCategory
		source: aText
		selection: anInterval! !

!OBMethodDefinition class methodsFor: 'instance creation' stamp: 'cwp 10/17/2004 23:07'!
selection: anInterval source: aText inClass: aClass
	^ self 
		selection: anInterval
		source: aText
		inCategory: nil
		inClass: aClass	! !

!OBMethodDefinition class methodsFor: 'instance creation' stamp: 'cwp 10/17/2004 23:03'!
source: aText inClass: aClass
	^ self 
		selection: nil
		source: aText
		inCategory: nil
		inClass: aClass! !

!OBMethodDefinition methodsFor: 'callbacks' stamp: 'cwp 9/18/2005 14:53'!
accept: aText notifying: aController
	| newSelector |
	newSelector := self compileMethod: aText notifying: aController.
	newSelector ifNil: [^ false].
	(self nodeFor: newSelector) demandSelection.
	^ true! !

!OBMethodDefinition methodsFor: 'accessing' stamp: 'cwp 10/17/2004 23:36'!
callback: aBlockContext 
	callback _ aBlockContext! !

!OBMethodDefinition methodsFor: 'accessing' stamp: ''!
category
	^ category ifNil: [category := self theClass whichCategoryIncludesSelector: self selector]! !

!OBMethodDefinition methodsFor: 'compiling' stamp: ''!
compileMethod: aText notifying: aController
	^ (self confirmMethod: aText)
		ifTrue: [self theClass 
					compile: aText 
					classified: self category 
					notifying: aController]
		ifFalse: [nil]! !

!OBMethodDefinition methodsFor: 'confirming' stamp: 'cwp 10/17/2004 22:42'!
confirmMethod: aText
	| sel |
	sel _ Parser new parseSelector: aText.
	^ (self theClass isMeta 
			and: [(self selectorAlreadyDefined: sel) not] 
			and: [Metaclass isScarySelector: sel])
				ifTrue: [self confirmScarySelector: sel]
				ifFalse: [^ true]! !

!OBMethodDefinition methodsFor: 'confirming' stamp: ''!
confirmScarySelector: aSelector
	| prompt |
	prompt := aSelector, ' is used in the existing class system.
Overriding it could cause serious problems.
Is this really what you want to do?'.

	^ OBConfirmationRequest
		prompt: prompt
		confirm: 'Override'! !

!OBMethodDefinition methodsFor: 'compiling' stamp: 'cwp 10/17/2004 23:33'!
nodeFor: aSelector
	^ callback value: aSelector! !

!OBMethodDefinition methodsFor: 'callbacks' stamp: 'cwp 10/17/2004 23:16'!
redefineSource: aText selector: newSelector
	newSelector = self selector 
		ifTrue: [source := aText]
		ifFalse: [(self methodNodeFor: newSelector) signalSelection].
! !

!OBMethodDefinition methodsFor: 'callbacks' stamp: ''!
selectedClass
	^ self theClass! !

!OBMethodDefinition methodsFor: 'callbacks' stamp: ''!
selection
	^ selection ifNil: [1 to: (source ifNil: [self text size] ifNotNil: [0])]! !

!OBMethodDefinition methodsFor: 'accessing' stamp: ''!
selector
	^ source ifNotNil: [Parser new parseSelector: source]! !

!OBMethodDefinition methodsFor: 'confirming' stamp: ''!
selectorAlreadyDefined: aSelector
	^ self theClass selectors includes: aSelector! !

!OBMethodDefinition methodsFor: 'initializing' stamp: 'cwp 10/17/2004 23:33'!
setClass: aClass category: aString source: aText selection: anInterval
	theClass := aClass.
	category := aString.
	source := aText.
	selection := anInterval.
	callback := [:sel | OBMethodNode on: sel inClass: theClass]! !

!OBMethodDefinition methodsFor: 'callbacks' stamp: 'cwp 11/29/2004 21:21'!
text
	^ source 
		ifNotNil: [source asText makeSelectorBold]
		ifNil: [self theClass sourceCodeTemplate]! !

!OBMethodDefinition methodsFor: 'accessing' stamp: ''!
theClass
	^ theClass! !

OBDefinition subclass: #OBOrganizationDefinition
	instanceVariableNames: 'organizer'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OB-Standard-Definitions'!

!OBOrganizationDefinition commentStamp: 'cwp 1/8/2005 13:31' prior: 0!
OBOrganizationDefinition knows how to display and edit the organization of classes or methods by a Categorizer.!

!OBOrganizationDefinition class methodsFor: 'as yet unclassified' stamp: 'cwp 9/22/2004 21:20'!
on: anOrganizer
	^ self new organizer: anOrganizer! !

!OBOrganizationDefinition methodsFor: 'as yet unclassified' stamp: 'cwp 9/22/2004 21:21'!
organizer: anOrganizer
	organizer _ anOrganizer! !

!OBOrganizationDefinition methodsFor: 'as yet unclassified' stamp: 'cwp 9/22/2004 21:18'!
text
	^ organizer asString! !

!OBOrganizationDefinition methodsFor: 'as yet unclassified' stamp: 'cwp 1/9/2005 11:45'!
text: aText
	organizer changeFromString: aText asString.
	^ true! !

OBTextPanel subclass: #OBAnnotationPanel
	instanceVariableNames: 'text'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OB-Standard-Panels'!

!OBAnnotationPanel methodsFor: 'callbacks' stamp: 'lr 5/2/2007 17:23'!
accept: aString notifying: aController
	^ false! !

!OBAnnotationPanel methodsFor: 'building' stamp: 'lr 5/2/2007 17:19'!
buildOn: aBuilder
	^ aBuilder mercuryPanel: self with: []! !

!OBAnnotationPanel methodsFor: 'callbacks' stamp: 'lr 5/2/2007 17:20'!
selection
	^ 1 to: 0! !

!OBAnnotationPanel methodsFor: 'updating' stamp: 'dc 5/2/2007 17:51'!
selectionChanged: ann
	text := ann node ifNil: [String new] ifNotNil: [ann node annotationString].
	self changed: #text! !

!OBAnnotationPanel methodsFor: 'updating' stamp: 'lr 5/2/2007 17:17'!
subscribe
	self announcer 
		observe: OBSelectionChanged
		send: #selectionChanged:
		to: self! !

!OBAnnotationPanel methodsFor: 'callbacks' stamp: 'lr 5/2/2007 17:21'!
text
	^ text! !

!OBAnnotationPanel methodsFor: 'building' stamp: 'lr 5/2/2007 17:19'!
vResizing
	^ #rigid! !

OBTextPanel subclass: #OBMercuryPanel
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OB-Standard-Panels'!

!OBMercuryPanel class methodsFor: 'class initialization' stamp: 'dc 5/8/2007 19:49'!
initialize
	"OBMercuryPanel initialize"
	Preferences 
		addPreference: #mercuryPanel
		categories: #('browsing')
		default: true
		balloonHelp: 'Adds the mercury panel to OmniBrowser. This panel is the one on top to search senders, implementors, classes...'! !

!OBMercuryPanel methodsFor: 'callbacks' stamp: 'cwp 4/19/2007 21:10'!
accept: aText notifying: anObject
	self execute: aText.
	self changed: #text! !

!OBMercuryPanel methodsFor: 'user interface' stamp: 'cwp 4/19/2007 22:30'!
addTextCommandsToMenu: aMenu selection: aTextSelection! !

!OBMercuryPanel methodsFor: 'actions' stamp: 'cwp 4/19/2007 00:06'!
browse: aSymbol
	(Smalltalk hasClassNamed: aSymbol)
		ifTrue: [self openClassBrowserOn: (OBClassNode on: (Smalltalk at: aSymbol))]
		ifFalse: [self findClass: aSymbol]
	! !

!OBMercuryPanel methodsFor: 'building' stamp: 'cwp 3/18/2007 20:56'!
buildOn: aBuilder
	^ aBuilder mercuryPanel: self with: []! !

!OBMercuryPanel methodsFor: 'actions' stamp: 'cwp 4/18/2007 23:03'!
classRefs: aSymbol
	(Smalltalk classNamed: aSymbol)
		ifNotNilDo: [:c | self openSearchBrowserOn: (OBClassRefSearchNode on: aSymbol)]! !

!OBMercuryPanel methodsFor: 'private' stamp: 'cwp 4/19/2007 21:57'!
environment
	^ Smalltalk! !

!OBMercuryPanel methodsFor: 'private' stamp: 'cwp 4/19/2007 21:09'!
execute: aText
	|refs literal tokens wildcard |
	tokens _ Scanner new scanTokens: aText asString.
	literal _ tokens at: 1 ifAbsent: [^ self].
	wildcard _ (tokens at: 2 ifAbsent: []) = #*.
	refs _ aText asString withBlanksTrimmed first = $#.
	literal isSymbol ifTrue:
		[literal first isUppercase ifTrue:
			[wildcard ifTrue:
				[self findClass: literal]
			ifFalse:
				[refs ifTrue: [self classRefs: literal] ifFalse: [self browse: literal]]]
		ifFalse:
			[wildcard ifTrue:
				[self findMethod: literal]
			ifFalse:
				[refs ifTrue: [self senders: literal] ifFalse: [self implementors: literal]]]]
	ifFalse: [self string: literal]! !

!OBMercuryPanel methodsFor: 'actions' stamp: 'cwp 4/19/2007 00:07'!
findClass: aSymbol
	self openSearchBrowserOn: (OBClassSearchNode on: aSymbol)! !

!OBMercuryPanel methodsFor: 'actions' stamp: 'cwp 4/19/2007 00:45'!
findMethod: aSymbol
	self openSearchBrowserOn: (OBMethodSearchNode on: aSymbol)! !

!OBMercuryPanel methodsFor: 'actions' stamp: 'cwp 4/19/2007 21:23'!
implementors: aSelector 
	(self systemNavigation allImplementedMessages includes: aSelector)
		ifTrue: [self openSearchBrowserOn: (OBImplementorSearchNode on: aSelector)]
		ifFalse: [self findMethod: aSelector]! !

!OBMercuryPanel methodsFor: 'private' stamp: 'cwp 4/18/2007 23:57'!
openClassBrowserOn: aNode
	^ self browser isClassBrowser
		ifTrue: [self browser jumpTo: aNode]
		ifFalse: [(OBSystemBrowser selection: aNode) open]! !

!OBMercuryPanel methodsFor: 'private' stamp: 'cwp 4/18/2007 23:03'!
openSearchBrowserOn: aNode
	^ self browser isSearchBrowser
		ifFalse: [(OBSearchBrowser with: aNode) open]
		ifTrue: 
			[self browser root addChild: aNode.
			(self announce: OBAboutToChange) isVetoed 
				ifTrue: [self browser noteChanges]
				ifFalse: [self announce: (OBSelectingNode node: aNode)]]! !

!OBMercuryPanel methodsFor: 'callbacks' stamp: 'cwp 2/25/2007 19:44'!
selection
	^ 1 to: 0! !

!OBMercuryPanel methodsFor: 'actions' stamp: 'cwp 4/18/2007 23:03'!
senders: aSelector
	self openSearchBrowserOn: (OBSenderSearchNode on: aSelector)! !

!OBMercuryPanel methodsFor: 'actions' stamp: 'cwp 3/3/2007 00:28'!
string: aString
	self systemNavigation browseMethodsWithString: aString! !

!OBMercuryPanel methodsFor: 'callbacks' stamp: 'cwp 2/25/2007 19:43'!
text
	^ ''! !

!OBMercuryPanel methodsFor: 'building' stamp: 'cwp 3/18/2007 20:54'!
vResizing
	^ #rigid! !

!OBTextPanel methodsFor: '*ob-standard-cmds' stamp: 'dc 5/2/2007 19:25'!
browseIt: aSymbol
	| entry |
	entry := self environment at: aSymbol ifAbsent: [nil].
	entry ifNil: [^ self implementorsOfIt: aSymbol].
	(entry isKindOf: Class) ifFalse: [entry _ entry class].
	OBSystemBrowser openOnClass: entry.
	^ true! !

!OBTextPanel methodsFor: '*ob-standard-cmds' stamp: 'cwp 4/19/2007 21:30'!
implementorsOfIt: aSelector 
	OBImplementorsBrowser browseRoot: (OBSelectorNode on: aSelector).
	^true! !

!OBTextPanel methodsFor: '*ob-standard-cmds' stamp: 'cwp 4/19/2007 21:54'!
referencesToIt: aClassName 
	| class |
	class := self environment at: aClassName ifAbsent: [^false].
	class isBehavior ifFalse: [^false].
	OBReferencesBrowser browseRoot: (OBClassNode on: class).
	^true! !

!OBTextPanel methodsFor: '*ob-standard-cmds' stamp: 'cwp 4/19/2007 21:30'!
sendersOfIt: aSelector 
	OBSendersBrowser browseRoot: (OBSelectorNode on: aSelector).
	^true! !

Object subclass: #OBAnnotationRequest
	instanceVariableNames: 'selector class separator'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OB-Standard-Utilities'!

!OBAnnotationRequest class methodsFor: 'instance-creation' stamp: 'dc 5/2/2007 18:10'!
onClass: aClass selector: aSelector
	^ super new
		onClass: aClass selector: aSelector;
		yourself! !

!OBAnnotationRequest methodsFor: 'requests' stamp: 'dc 5/2/2007 18:41'!
allChangeSets
	^ (ChangeSorter allChangeSetsWithClass: class selector: selector) 
		ifNotEmpty: 
			[String streamContents: 
					[:aStream | 
					(ChangeSorter allChangeSetsWithClass: class selector: selector) size = 1 
						ifTrue: [aStream nextPutAll: 'only in change set ']
						ifFalse: [aStream nextPutAll: 'in change sets: '].
					(ChangeSorter allChangeSetsWithClass: class selector: selector) 
						do: [:aChangeSet | aStream nextPutAll: aChangeSet name , ' ']]]
		ifEmpty: ['in no change set']! !

!OBAnnotationRequest methodsFor: 'actions' stamp: 'dc 5/2/2007 18:48'!
annotationRequests
	^ Preferences defaultAnnotationRequests! !

!OBAnnotationRequest methodsFor: 'requests' stamp: 'dc 5/2/2007 18:57'!
documentation
	^ class precodeCommentOrInheritedCommentFor: selector! !

!OBAnnotationRequest methodsFor: 'requests' stamp: 'dc 5/2/2007 18:20'!
firstComment
	^ class firstCommentAt: selector! !

!OBAnnotationRequest methodsFor: 'actions' stamp: 'avi 5/30/2007 23:15'!
getAnnotations
	^ String streamContents: [:aStream |
		
			(self annotationRequests
				collect: [:request | self perform: request]
				thenSelect: [:stringOrNil | stringOrNil isEmptyOrNil not])
				do: [:ea | aStream nextPutAll: ea] separatedBy: [aStream nextPutAll: separator]]! !

!OBAnnotationRequest methodsFor: 'requests' stamp: 'dc 5/2/2007 18:53'!
implementorsCount
	|implementorsCount|

	implementorsCount := self systemNavigation numberOfImplementorsOf: selector.
	implementorsCount := implementorsCount == 1
							ifTrue: ['1 implementor']
							ifFalse: [implementorsCount printString , ' implementors'].
	^ implementorsCount ! !

!OBAnnotationRequest methodsFor: 'requests' stamp: 'dc 5/2/2007 18:57'!
masterComment
	^ class supermostPrecodeCommentFor: selector! !

!OBAnnotationRequest methodsFor: 'requests' stamp: 'dc 5/2/2007 18:53'!
messageCategory
	^ class organization categoryOfElement: selector! !

!OBAnnotationRequest methodsFor: 'accessing' stamp: 'dc 6/18/2007 10:08'!
method
	^ class compiledMethodAt: selector ifAbsent: nil! !

!OBAnnotationRequest methodsFor: 'initialize-release' stamp: 'avi 5/30/2007 23:16'!
onClass: aClass selector: aSelector
	class := aClass.
	selector := aSelector.
	separator := '  '.! !

!OBAnnotationRequest methodsFor: 'requests' stamp: 'dc 5/2/2007 19:00'!
priorTimeStamp
	^ (VersionsBrowser 
		timeStampFor: selector
		class: class
		reverseOrdinal: 2) ifNotNilDo: [:stamp | 'prior time stamp: ' , stamp]! !

!OBAnnotationRequest methodsFor: 'requests' stamp: 'dc 5/2/2007 18:34'!
priorVersionsCount
	| versionsCount |

	versionsCount := VersionsBrowser versionCountForSelector: selector class: class.
	^ versionsCount > 1
			ifTrue:
				[versionsCount == 2
					ifTrue: ['1 prior version']
					ifFalse: [versionsCount printString, ' prior versions']]
			ifFalse:
				['no prior versions']! !

!OBAnnotationRequest methodsFor: 'requests' stamp: 'dc 5/2/2007 18:37'!
recentChangeSet
	^ ChangeSorter mostRecentChangeSetWithChangeForClass: class
		selector: selector! !

!OBAnnotationRequest methodsFor: 'requests' stamp: 'dc 5/2/2007 18:31'!
sendersCount
	| sendersCount |
	sendersCount := (self systemNavigation allCallsOn: selector) size.
	sendersCount := sendersCount == 1
						ifTrue: ['1 sender']
						ifFalse: [sendersCount printString , ' senders'].
	^ sendersCount! !

!OBAnnotationRequest methodsFor: 'accessing' stamp: 'dc 5/2/2007 18:14'!
separator: aString
	separator := aString! !

!OBAnnotationRequest methodsFor: 'requests' stamp: 'dc 5/2/2007 18:29'!
timeStamp
	^ self method ifNotNilDo: [:method | Utilities timeStampForMethod: method]! !

Object subclass: #OBClassReference
	instanceVariableNames: 'name isMeta'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OB-Standard-Utilities'!

!OBClassReference commentStamp: 'cwp 1/8/2005 13:15' prior: 0!
OBClassReference provides a way to refer to classes that may or may not be loaded into the image. It refers to the class indirectly, via name, rather than with a direct pointer to the class object. It also provides a number of convenience methods, which makes it more convenient than using the class name directly.!

!OBClassReference class methodsFor: 'instance creation' stamp: ''!
metaNamed: aSymbol
	^ (self named: aSymbol) beMeta! !

!OBClassReference class methodsFor: 'instance creation' stamp: ''!
named: aSymbol
	^ self new setName: aSymbol! !

!OBClassReference class methodsFor: 'instance creation' stamp: ''!
to: aClass
	^ self named: aClass name! !

!OBClassReference methodsFor: 'comparing' stamp: ''!
<= other
	^ self name <= other name! !

!OBClassReference methodsFor: 'comparing' stamp: ''!
= other
	^ (other isKindOf: self class) 
		and: [name = other theNonMetaName] 
		and: [isMeta = other isMeta]! !

!OBClassReference methodsFor: 'accessing' stamp: ''!
beMeta
	isMeta := true! !

!OBClassReference methodsFor: 'accessing' stamp: ''!
beNonMeta
	isMeta := false! !

!OBClassReference methodsFor: 'comparing' stamp: ''!
hash
	^ isMeta
		ifTrue: [name hash bitInvert]
		ifFalse: [name hash]! !

!OBClassReference methodsFor: 'testing' stamp: ''!
isMeta
	^ isMeta! !

!OBClassReference methodsFor: 'accessing' stamp: ''!
name
	^ isMeta
		ifTrue: [name, ' class']
		ifFalse: [name]! !

!OBClassReference methodsFor: 'printing' stamp: ''!
printOn: aStream
	aStream nextPutAll: 'OBClassReference'.
	aStream nextPut: $<.
	aStream nextPutAll: name.
	isMeta ifTrue: [aStream nextPutAll: ' class'].
	aStream nextPut: $>.! !

!OBClassReference methodsFor: 'converting' stamp: ''!
referenceToMethod: aSelector
	| ref |
	ref := MethodReference new.
	ref setClassSymbol: name classIsMeta: isMeta  methodSymbol: aSelector stringVersion: ''.
	^ ref! !

!OBClassReference methodsFor: 'initializing' stamp: ''!
setName: aSymbol
	name := (aSymbol copyUpTo: $ ) asSymbol.
	isMeta := aSymbol endsWith: ' class'.! !

!OBClassReference methodsFor: 'accessing' stamp: ''!
theClass
	| theClass |
	theClass := self theNonMetaClass ifNil: [^ nil].
	^ isMeta
		ifFalse: [theClass]
		ifTrue: [theClass class]! !

!OBClassReference methodsFor: 'accessing' stamp: ''!
theNonMetaClass
	^ Smalltalk at: name ifAbsent: [].! !

!OBClassReference methodsFor: 'accessing' stamp: ''!
theNonMetaName
	^ name! !

!OBClassReference methodsFor: 'accessing' stamp: ''!
theNonMetaName: aSymbol
	name := aSymbol! !

Object subclass: #OBMethodVersion
	instanceVariableNames: 'sources pointer classRef className category stamp prior selector'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OB-Standard-Utilities'!

!OBMethodVersion commentStamp: 'cwp 1/8/2005 13:25' prior: 0!
OBMethodVerison represents a version of a method found in the .sources or .changes files. It provides methods for parsing the method definition referenced by a source pointer, and for filing in the source to replace the current version of the method.!

!OBMethodVersion class methodsFor: 'instance creation' stamp: ''!
fromSources: sources andPointer: pointer
	^ self new setSources: sources pointer: pointer! !

!OBMethodVersion class methodsFor: 'scanning' stamp: ''!
scan: sources from: pointer 
	| versions current |
	
	versions := OrderedCollection new.
	current := OBMethodVersion fromSources: sources andPointer: pointer.
	[current notNil]
		whileTrue: [versions add: current.
					current := current previous].
	^ versions! !

!OBMethodVersion class methodsFor: 'scanning' stamp: ''!
versionsOfMethod: methodReference 
	| class selector |
	class := methodReference actualClass.
	selector := methodReference methodSymbol.
	^ self scan: SourceFiles from: (class compiledMethodAt: selector) sourcePointer! !

!OBMethodVersion methodsFor: 'comparing' stamp: 'cwp 10/17/2004 23:45'!
= other
	^ self species = other species and: [self pointer = other pointer]! !

!OBMethodVersion methodsFor: 'accessing' stamp: ''!
category
	^ category! !

!OBMethodVersion methodsFor: 'accessing' stamp: ''!
classSymbol
	^ classRef name! !

!OBMethodVersion methodsFor: 'compiling' stamp: ''!
fileIn
	(self theClass) ifNotNilDo: [:class | class
						 				compile: self source 
										classified: self category 
										withStamp: self stamp 
										notifying: nil]
						
						! !

!OBMethodVersion methodsFor: 'comparing' stamp: 'cwp 10/17/2004 23:45'!
hash
	^ pointer hash! !

!OBMethodVersion methodsFor: 'accessing' stamp: 'cwp 10/17/2004 23:51'!
latest
	^ self class 
			fromSources: sources 
			andPointer: (classRef theClass compiledMethodAt: selector) sourcePointer! !

!OBMethodVersion methodsFor: 'initializing' stamp: 'cwp 11/6/2004 23:14'!
parseChunk: aString
	| tokens |
	tokens := Scanner new scanTokens: aString.
	classRef := OBClassReference named: tokens first.
	tokens second = #class
		ifTrue: [classRef beMeta.
				category := tokens fourth.
				stamp := tokens sixth]
		ifFalse: [category := tokens third.
				tokens size > 3 ifTrue: [stamp := tokens fifth]].
	tokens size > 6 ifTrue: [prior := tokens last].! !

!OBMethodVersion methodsFor: 'initializing' stamp: ''!
parseSource
	| file position chunk |
	file := sources at: (sources fileIndexFromSourcePointer: pointer).
	position := sources filePositionFromSourcePointer: pointer.
	position > file size ifTrue: [self error: 'Invalid source pointer'].
	
	file position: (0 max: position-150).  	"Skip back to before the preamble"
		[file position < (position-1)]  	"then pick it up from the front"
			whileTrue: [chunk := file nextChunk].
	
	self parseChunk: chunk.! !

!OBMethodVersion methodsFor: 'accessing' stamp: 'cwp 10/17/2004 23:42'!
pointer
	^ pointer! !

!OBMethodVersion methodsFor: 'initializing' stamp: ''!
previous
	^ prior ifNotNil: [self class fromSources: sources andPointer: prior]! !

!OBMethodVersion methodsFor: 'accessing' stamp: ''!
selector
	^ selector ifNil: [self setSelector]! !

!OBMethodVersion methodsFor: 'initializing' stamp: ''!
setSelector
	| file |
	file := sources at: (sources fileIndexFromSourcePointer: pointer).
	file position: (sources filePositionFromSourcePointer: pointer).
	^ selector := Parser new parseSelector: file nextChunk.! !

!OBMethodVersion methodsFor: 'initializing' stamp: ''!
setSources: aSourceFileArray pointer: aSourcePointer
	sources := aSourceFileArray.
	pointer := aSourcePointer.
	self parseSource.! !

!OBMethodVersion methodsFor: 'accessing' stamp: ''!
source
	| file |
	file := sources at: (sources fileIndexFromSourcePointer: pointer).
	file position: (sources filePositionFromSourcePointer: pointer).
	^ file nextChunk asText makeSelectorBold! !

!OBMethodVersion methodsFor: 'accessing' stamp: ''!
stamp
	^ stamp! !

!OBMethodVersion methodsFor: 'accessing' stamp: ''!
theClass
	^ classRef theClass! !

!OBMethodVersion methodsFor: 'accessing' stamp: ''!
theClassName
	^ classRef name! !

Object subclass: #OBSystemBrowserAdaptor
	instanceVariableNames: 'class selector'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OB-Standard-Utilities'!

!OBSystemBrowserAdaptor commentStamp: 'cwp 1/8/2005 13:35' prior: 0!
OBSystemBrowserAdaptor implements the protocol expected by the SystemBrowser app registry, and thus allows OmniBrowser to be chosen as the default system browser. 

Caveat: because the required protocol is a little ...odd.... it can't be implemented well by OBSystemBrowser directly. OBSystemBrowserAdaptor does this reasonably well, but it has no way to provide a more natural name than its class name to the app registry menu.!

!OBSystemBrowserAdaptor class methodsFor: 'registry protocol' stamp: 'lr 4/20/2006 08:48'!
fullOnClass: aClass selector: aSelector
	^ OBSystemBrowser 
		openOnClass: aClass
		selector: aSelector! !

!OBSystemBrowserAdaptor class methodsFor: 'initializing' stamp: 'cwp 12/5/2004 22:01'!
initialize
	self register! !

!OBSystemBrowserAdaptor class methodsFor: 'registry protocol' stamp: 'lr 3/28/2006 09:10'!
open
	OBSystemBrowser open! !

!OBSystemBrowserAdaptor class methodsFor: 'registry protocol' stamp: 'cwp 12/5/2004 21:59'!
openBrowser
	OBSystemBrowser open! !

!OBSystemBrowserAdaptor class methodsFor: 'registry protocol' stamp: 'cwp 12/5/2004 21:56'!
openBrowserView: adaptor label: ignored 
	^ OBSystemBrowser 
		openOnClass: adaptor targetClass 
		selector: adaptor selector! !

!OBSystemBrowserAdaptor class methodsFor: 'initializing' stamp: 'cwp 1/8/2005 21:52'!
register
	Smalltalk at: #SystemBrowser ifPresent: [:class | class register: self]! !

!OBSystemBrowserAdaptor class methodsFor: 'initializing' stamp: 'cwp 1/8/2005 21:53'!
unregister
	Smalltalk at: #SystemBrowser ifPresent: [:class | class unregister: self]! !

!OBSystemBrowserAdaptor methodsFor: 'registry protocol' stamp: 'cwp 12/5/2004 21:50'!
labelString
	^ 'System Browser'! !

!OBSystemBrowserAdaptor methodsFor: 'registry protocol' stamp: 'cwp 12/5/2004 21:49'!
openEditString: anUndefinedObject 
	^ self! !

!OBSystemBrowserAdaptor methodsFor: 'accessing' stamp: 'cwp 12/5/2004 21:55'!
selector
	^ selector! !

!OBSystemBrowserAdaptor methodsFor: 'registry protocol' stamp: 'cwp 12/5/2004 21:49'!
setClass: aClass selector: aSelector 
	class _ aClass.
	selector _ aSelector! !

!OBSystemBrowserAdaptor methodsFor: 'accessing' stamp: 'cwp 12/5/2004 21:55'!
targetClass
	^ class! !

!ClassDescription methodsFor: '*ob-standard-converting' stamp: 'dvf 8/17/2005 17:31'!
asCommentNode
	^ OBClassCommentNode on: self theNonMetaClass 
! !

OBCommand subclass: #OBCategoryCommand
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OB-Standard-Commands'!

!OBCategoryCommand methodsFor: 'as yet unclassified' stamp: 'avi 5/30/2007 23:07'!
group
	^ #category! !

!OBCategoryCommand methodsFor: 'as yet unclassified' stamp: 'avi 5/30/2007 23:11'!
isActive
	^ target hasOrganization
		and: [(requestor isSelected: target) not]
		and: [requestor selectedNode isNil or: [requestor selectedNode isCategory]]! !

!OBCategoryCommand methodsFor: 'as yet unclassified' stamp: 'avi 5/30/2007 23:07'!
label
	^ 'sort categories'! !

OBCategoryCommand subclass: #OBCmdCreateCategory
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OB-Standard-Commands'!

!OBCmdCreateCategory methodsFor: 'private' stamp: 'cwp 9/25/2006 00:12'!
categoryTemplateFor: anOrganization
	^ anOrganization isClassOrganizer
		ifTrue: ['category name']
		ifFalse: ['Category-Name']! !

!OBCmdCreateCategory methodsFor: 'public' stamp: 'cwp 10/15/2006 19:01'!
execute
	| organization catName |
	organization _ target organization.
	catName _ OBTextRequest 
					prompt: 'Please type new category name' 
					template: (self categoryTemplateFor: organization).
	catName ifNotNil:	[organization addCategory: catName.
						self 
							select: (target categoryNodeNamed: catName) 
							with: requestor announcer].! !

!OBCmdCreateCategory methodsFor: 'public' stamp: 'lr 6/9/2007 10:33'!
icon
	^ #newIcon! !

!OBCmdCreateCategory methodsFor: 'public' stamp: 'cwp 9/25/2006 00:10'!
label
	^ 'create category...'! !

OBCategoryCommand subclass: #OBCmdRemoveEmptyCats
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OB-Standard-Commands'!

!OBCmdRemoveEmptyCats methodsFor: 'as yet unclassified' stamp: 'cwp 10/15/2006 18:52'!
execute
	target organization removeEmptyCategories! !

!OBCmdRemoveEmptyCats methodsFor: 'as yet unclassified' stamp: 'cwp 9/28/2006 08:52'!
label
	^ 'remove empty categories'! !

OBCategoryCommand subclass: #OBCmdReorgCats
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OB-Standard-Commands'!

!OBCmdReorgCats methodsFor: 'as yet unclassified' stamp: 'cwp 10/15/2006 18:53'!
execute
	| definition organizer |
	organizer _ target organization.
	definition _ OBOrganizationDefinition on: organizer.
	requestor announce: definition! !

!OBCmdReorgCats methodsFor: 'as yet unclassified' stamp: 'cwp 9/28/2006 09:12'!
label
	^ 'reorganize categories'! !

OBCategoryCommand subclass: #OBCmdSortCats
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OB-Standard-Commands'!

!OBCmdSortCats methodsFor: 'as yet unclassified' stamp: 'cwp 10/15/2006 18:41'!
execute
	target organization sortCategories.
	requestor announce: (OBChildrenChanged node: target)! !

!OBCmdSortCats methodsFor: 'as yet unclassified' stamp: 'cwp 9/28/2006 09:24'!
label
	^ 'sort categories'! !

OBCommand subclass: #OBCmdAutoCategorize
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OB-Standard-Commands'!

!OBCmdAutoCategorize methodsFor: 'as yet unclassified' stamp: 'cwp 10/15/2006 01:12'!
execute

	"Categorize methods by looking in parent classes for a method category."
	| organizers organizer |
	organizer _ target organization.
	organizers := organizer subject withAllSuperclasses collect: [:ea | ea organization].
	(organizer listAtCategoryNamed: ClassOrganizer default) do: [:sel | | found |
		found := (organizers collect: [ :org | org categoryOfElement: sel])
			detect: [:ea | ea ~= ClassOrganizer default and: [ ea ~= nil]]
			ifNone: [].
		found ifNotNil: [organizer classify: sel under: found]]! !

!OBCmdAutoCategorize methodsFor: 'as yet unclassified' stamp: 'cwp 10/9/2006 15:07'!
group
	^ #category! !

!OBCmdAutoCategorize methodsFor: 'as yet unclassified' stamp: 'cwp 10/9/2006 15:57'!
isActive
	^ target hasOrganization 
		and: [target organization isClassOrganizer]
		and: [(requestor isSelected: target) not]! !

!OBCmdAutoCategorize methodsFor: 'as yet unclassified' stamp: 'cwp 9/25/2006 00:25'!
label
	^ 'categorize automatically'! !

OBCommand subclass: #OBCmdBrowse
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OB-Standard-Commands'!

!OBCmdBrowse methodsFor: 'as yet unclassified' stamp: 'cwp 10/15/2006 18:48'!
execute
	target browse! !

!OBCmdBrowse methodsFor: 'as yet unclassified' stamp: 'cwp 10/9/2006 17:56'!
group
	^ #navigation! !

!OBCmdBrowse methodsFor: 'as yet unclassified' stamp: 'cwp 10/9/2006 15:44'!
isActive
	^ (requestor isSelected: target)
		and: [target respondsTo: #browse]
		
		
! !

!OBCmdBrowse methodsFor: 'as yet unclassified' stamp: 'cwp 10/9/2006 15:18'!
keystroke
	^ $b! !

!OBCmdBrowse methodsFor: 'as yet unclassified' stamp: 'cwp 10/9/2006 15:17'!
label
	^ 'browse'! !

!OBCmdBrowse methodsFor: 'as yet unclassified' stamp: 'cwp 3/11/2007 19:34'!
wantsButton
	^ true! !

OBCommand subclass: #OBCmdBrowseHierarchy
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OB-Standard-Commands'!

!OBCmdBrowseHierarchy methodsFor: 'as yet unclassified' stamp: 'lr 4/25/2007 09:28'!
buttonLabel
	^ 'hierarchy'! !

!OBCmdBrowseHierarchy methodsFor: 'as yet unclassified' stamp: 'cwp 10/15/2006 18:48'!
execute
	OBHierarchyBrowser openOnClass: target theClass ! !

!OBCmdBrowseHierarchy methodsFor: 'as yet unclassified' stamp: 'cwp 10/11/2006 00:17'!
group
	^ #navigation! !

!OBCmdBrowseHierarchy methodsFor: 'as yet unclassified' stamp: 'lr 4/25/2007 09:33'!
isActive
	^ (target isKindOf: OBClassAwareNode)
		and: [requestor isSelected: target]! !

!OBCmdBrowseHierarchy methodsFor: 'as yet unclassified' stamp: 'cwp 10/11/2006 00:05'!
label
	^ 'browse hierarchy'! !

!OBCmdBrowseHierarchy methodsFor: 'as yet unclassified' stamp: 'lr 4/25/2007 09:40'!
wantsButton
	^ (target isKindOf: OBMethodVersionNode) not! !

OBCommand subclass: #OBCmdBrowseImplementors
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OB-Standard-Commands'!

!OBCmdBrowseImplementors methodsFor: 'as yet unclassified' stamp: 'cwp 3/11/2007 21:30'!
execute
	OBImplementorsBrowser browseRoot: target copy! !

!OBCmdBrowseImplementors methodsFor: 'as yet unclassified' stamp: 'cwp 10/15/2006 21:29'!
group
	^ #navigation! !

!OBCmdBrowseImplementors methodsFor: 'as yet unclassified' stamp: 'cwp 10/15/2006 21:13'!
isActive
	^ target hasSelector and: [requestor isSelected: target]! !

!OBCmdBrowseImplementors methodsFor: 'as yet unclassified' stamp: 'cwp 10/15/2006 21:13'!
keystroke
	^ $m! !

!OBCmdBrowseImplementors methodsFor: 'as yet unclassified' stamp: 'cwp 10/15/2006 21:09'!
label
	^ 'implementors'! !

!OBCmdBrowseImplementors methodsFor: 'as yet unclassified' stamp: 'lr 4/25/2007 09:40'!
wantsButton
	^ (target isKindOf: OBMethodVersionNode) not! !

OBCommand subclass: #OBCmdBrowseImplementorsOfIt
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OB-Standard-Commands'!

!OBCmdBrowseImplementorsOfIt class methodsFor: 'as yet unclassified' stamp: 'cwp 10/16/2006 22:13'!
takesNodes
	^ false! !

!OBCmdBrowseImplementorsOfIt class methodsFor: 'as yet unclassified' stamp: 'cwp 10/16/2006 22:01'!
takesText
	^ true! !

!OBCmdBrowseImplementorsOfIt methodsFor: 'as yet unclassified' stamp: 'cwp 10/16/2006 21:38'!
execute
	OBImplementorsBrowser browseRoot: (OBSelectorNode on: target selector)! !

!OBCmdBrowseImplementorsOfIt methodsFor: 'as yet unclassified' stamp: 'cwp 10/16/2006 21:32'!
isActive
	^ true! !

!OBCmdBrowseImplementorsOfIt methodsFor: 'as yet unclassified' stamp: 'cwp 10/16/2006 23:14'!
isEnabled
	^ target hasSelector! !

!OBCmdBrowseImplementorsOfIt methodsFor: 'as yet unclassified' stamp: 'cwp 3/11/2007 19:53'!
keystroke
	^ $m! !

!OBCmdBrowseImplementorsOfIt methodsFor: 'as yet unclassified' stamp: 'cwp 3/11/2007 19:52'!
label
	^ 'implementors'! !

OBCommand subclass: #OBCmdBrowseInheritance
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OB-Standard-Commands'!

!OBCmdBrowseInheritance methodsFor: 'as yet unclassified' stamp: 'cwp 10/17/2006 21:28'!
execute
	OBInheritanceBrowser openRoot: target inheritanceRoot selection: target! !

!OBCmdBrowseInheritance methodsFor: 'as yet unclassified' stamp: 'cwp 10/17/2006 21:30'!
group
	^ #navigation! !

!OBCmdBrowseInheritance methodsFor: 'as yet unclassified' stamp: 'cwp 10/17/2006 21:15'!
isActive
	^ target hasSelector and: [requestor isSelected: target]! !

!OBCmdBrowseInheritance methodsFor: 'as yet unclassified' stamp: 'cwp 10/17/2006 21:31'!
label
	^ 'inheritance'! !

!OBCmdBrowseInheritance methodsFor: 'as yet unclassified' stamp: 'lr 4/25/2007 09:40'!
wantsButton
	^ (target isKindOf: OBMethodVersionNode) not! !

OBCommand subclass: #OBCmdBrowseMethodVersions
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OB-Standard-Commands'!

!OBCmdBrowseMethodVersions methodsFor: 'as yet unclassified' stamp: 'cwp 10/15/2006 18:49'!
execute
	OBVersionBrowser openOn: target copy! !

!OBCmdBrowseMethodVersions methodsFor: 'as yet unclassified' stamp: 'cwp 10/9/2006 18:09'!
group
	^ #navigation! !

!OBCmdBrowseMethodVersions methodsFor: 'as yet unclassified' stamp: 'cwp 10/9/2006 18:16'!
isActive
	^ target hasVersions and: [requestor isSelected: target]! !

!OBCmdBrowseMethodVersions methodsFor: 'as yet unclassified' stamp: 'cwp 3/11/2007 19:53'!
keystroke
	^ $v! !

!OBCmdBrowseMethodVersions methodsFor: 'as yet unclassified' stamp: 'cwp 10/9/2006 18:12'!
label
	^ 'versions'! !

!OBCmdBrowseMethodVersions methodsFor: 'as yet unclassified' stamp: 'lr 4/25/2007 09:40'!
wantsButton
	^ (target isKindOf: OBMethodVersionNode) not! !

OBCommand subclass: #OBCmdBrowseMethodsWithIt
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OB-Standard-Commands'!

!OBCmdBrowseMethodsWithIt class methodsFor: 'as yet unclassified' stamp: 'DM 5/23/2007 16:39'!
takesNodes
	^ false! !

!OBCmdBrowseMethodsWithIt class methodsFor: 'as yet unclassified' stamp: 'DM 5/23/2007 16:39'!
takesText
	^ true! !

!OBCmdBrowseMethodsWithIt methodsFor: 'as yet unclassified' stamp: 'DM 5/23/2007 20:54'!
execute
	OBMethodStringsBrowser browseRoot: (OBSelectorNode on: target text)
		target: requestor browser currentNode! !

!OBCmdBrowseMethodsWithIt methodsFor: 'as yet unclassified' stamp: 'DM 5/23/2007 16:32'!
isActive
	^ true.! !

!OBCmdBrowseMethodsWithIt methodsFor: 'as yet unclassified' stamp: 'DM 5/23/2007 16:40'!
isEnabled
	^ true! !

!OBCmdBrowseMethodsWithIt methodsFor: 'as yet unclassified' stamp: 'DM 5/23/2007 20:51'!
keystroke
	^ $E.! !

!OBCmdBrowseMethodsWithIt methodsFor: 'as yet unclassified' stamp: 'DM 5/23/2007 16:40'!
label
	^ 'methods containing'! !

OBCommand subclass: #OBCmdBrowseProtocol
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OB-Standard-Commands'!

!OBCmdBrowseProtocol methodsFor: 'execution' stamp: 'DM 5/20/2007 15:17'!
execute
	OBProtocolBrowser on: target.! !

!OBCmdBrowseProtocol methodsFor: 'accessing' stamp: 'jk 5/20/2007 00:00'!
group
	^ #navigation! !

!OBCmdBrowseProtocol methodsFor: 'testing' stamp: 'jk 5/20/2007 00:02'!
isActive
	^ (target isKindOf: OBClassAwareNode )
		and: [requestor isSelected: target]! !

!OBCmdBrowseProtocol methodsFor: 'accessing' stamp: 'DM 5/20/2007 15:42'!
keystroke
	^ $P
	! !

!OBCmdBrowseProtocol methodsFor: 'accessing' stamp: 'DM 5/20/2007 15:38'!
label
	^ 'browse protocol'! !

OBCommand subclass: #OBCmdBrowseReferences
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OB-Standard-Commands'!

!OBCmdBrowseReferences methodsFor: 'as yet unclassified' stamp: 'cwp 12/7/2006 22:05'!
execute
	OBReferencesBrowser browseRoot: target theNonMetaClass asNode! !

!OBCmdBrowseReferences methodsFor: 'as yet unclassified' stamp: 'cwp 12/7/2006 22:30'!
group
	^ #navigation! !

!OBCmdBrowseReferences methodsFor: 'as yet unclassified' stamp: 'cwp 12/7/2006 21:51'!
isActive
	^ (target isKindOf: OBClassNode) and: [requestor isSelected: target]! !

!OBCmdBrowseReferences methodsFor: 'as yet unclassified' stamp: 'cwp 12/7/2006 21:43'!
label
	^ 'browse references'! !

OBCommand subclass: #OBCmdBrowseSenders
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OB-Standard-Commands'!

!OBCmdBrowseSenders methodsFor: 'as yet unclassified' stamp: 'cwp 10/15/2006 18:49'!
execute
	OBSendersBrowser browseRoot: target copy! !

!OBCmdBrowseSenders methodsFor: 'as yet unclassified' stamp: 'cwp 10/11/2006 21:29'!
group
	^ #navigation! !

!OBCmdBrowseSenders methodsFor: 'as yet unclassified' stamp: 'cwp 10/11/2006 21:45'!
isActive
	^ target hasSelector and: [requestor isSelected: target]! !

!OBCmdBrowseSenders methodsFor: 'as yet unclassified' stamp: 'cwp 10/11/2006 20:35'!
keystroke
	^ $n! !

!OBCmdBrowseSenders methodsFor: 'as yet unclassified' stamp: 'cwp 10/11/2006 20:32'!
label
	^ 'senders'! !

!OBCmdBrowseSenders methodsFor: 'as yet unclassified' stamp: 'lr 4/25/2007 09:40'!
wantsButton
	^ (target isKindOf: OBMethodVersionNode) not! !

OBCommand subclass: #OBCmdBrowseSendersOfIt
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OB-Standard-Commands'!

!OBCmdBrowseSendersOfIt class methodsFor: 'as yet unclassified' stamp: 'cwp 10/15/2006 01:00'!
takesNodes
	^ false! !

!OBCmdBrowseSendersOfIt class methodsFor: 'as yet unclassified' stamp: 'cwp 10/15/2006 01:00'!
takesText
	^ true! !

!OBCmdBrowseSendersOfIt methodsFor: 'as yet unclassified' stamp: 'cwp 10/15/2006 18:49'!
execute
	OBSendersBrowser browseRoot: (OBSelectorNode on: target selector)! !

!OBCmdBrowseSendersOfIt methodsFor: 'as yet unclassified' stamp: 'cwp 10/14/2006 21:06'!
isActive
	^ true! !

!OBCmdBrowseSendersOfIt methodsFor: 'as yet unclassified' stamp: 'cwp 10/15/2006 19:31'!
isEnabled
	^ target hasSelector! !

!OBCmdBrowseSendersOfIt methodsFor: 'as yet unclassified' stamp: 'cwp 3/11/2007 19:57'!
keystroke
	^ $n! !

!OBCmdBrowseSendersOfIt methodsFor: 'as yet unclassified' stamp: 'cwp 3/11/2007 19:57'!
label
	^ 'senders'! !

OBCommand subclass: #OBCmdChaseVariables
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OB-Standard-Commands'!

!OBCmdChaseVariables methodsFor: 'as yet unclassified' stamp: 'lr 4/25/2007 09:28'!
buttonLabel
	^ 'variables'! !

!OBCmdChaseVariables methodsFor: 'as yet unclassified' stamp: 'cwp 10/15/2006 18:49'!
execute
	^ OBVariablesBrowser browseRoot: target classNode! !

!OBCmdChaseVariables methodsFor: 'as yet unclassified' stamp: 'cwp 10/10/2006 22:50'!
group
	^ #navigation! !

!OBCmdChaseVariables methodsFor: 'as yet unclassified' stamp: 'lr 4/25/2007 09:42'!
isActive
	^ (target isKindOf: OBClassAwareNode)
		and: [requestor isSelected: target]! !

!OBCmdChaseVariables methodsFor: 'as yet unclassified' stamp: 'cwp 10/10/2006 22:20'!
label
	^ 'chase variables'! !

!OBCmdChaseVariables methodsFor: 'as yet unclassified' stamp: 'lr 4/25/2007 09:40'!
wantsButton
	^ (target isKindOf: OBMethodVersionNode) not! !

OBCommand subclass: #OBCmdCopyClass
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OB-Standard-Commands'!

!OBCmdCopyClass methodsFor: 'as yet unclassified' stamp: 'cwp 11/2/2006 21:37'!
copyClass: oldClass as: newName
	| oldDefinition newDefinition newClass |
	(oldClass environment hasClassNamed: newName)
		ifTrue: [^self error: newName, ' already exists'].
	oldDefinition := oldClass definition.
	newDefinition := oldDefinition 
						copyReplaceAll: '#' , oldClass name asString 
						with: '#' , newName asString.
	Cursor wait 
		showWhile: [newClass := Compiler evaluate: newDefinition logged: true.
					newClass copyAllCategoriesFrom: oldClass.
					newClass class copyAllCategoriesFrom: oldClass class].
	^ newClass! !

!OBCmdCopyClass methodsFor: 'as yet unclassified' stamp: 'cwp 11/2/2006 21:36'!
execute

	| newName newClass |
	newName := OBTextRequest
			prompt: 'Please type new class name'
			template: target theNonMetaClassName asString.
	newName ifNotNil:	
		[newClass := self
						copyClass: target theNonMetaClass 
						as: newName asSymbol.
		self select: newClass asNode with: requestor announcer].! !

!OBCmdCopyClass methodsFor: 'as yet unclassified' stamp: 'cwp 11/2/2006 22:00'!
group
	^ #identity! !

!OBCmdCopyClass methodsFor: 'as yet unclassified' stamp: 'cwp 11/2/2006 20:43'!
isActive
	^ (target isKindOf: OBClassNode)
		and: [requestor isSelected: target]
		! !

!OBCmdCopyClass methodsFor: 'as yet unclassified' stamp: 'cwp 11/2/2006 20:06'!
label
	^ 'copy...'! !

OBCommand subclass: #OBCmdFileOut
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OB-Standard-Commands'!

!OBCmdFileOut methodsFor: 'as yet unclassified' stamp: 'cwp 10/15/2006 18:50'!
execute
	target fileOut! !

!OBCmdFileOut methodsFor: 'as yet unclassified' stamp: 'cwp 10/9/2006 15:43'!
isActive
	^ (requestor isSelected: target) 
		and: [target respondsTo: #fileOut]! !

!OBCmdFileOut methodsFor: 'as yet unclassified' stamp: 'cwp 10/7/2006 11:19'!
label
	^ 'file out'! !

OBCommand subclass: #OBCmdFindClass
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OB-Standard-Commands'!

!OBCmdFindClass methodsFor: 'private' stamp: 'dc 5/17/2007 12:10'!
allBehaviorsIn: anEnvironment 
	^ anEnvironment classNames asArray! !

!OBCmdFindClass methodsFor: 'execution' stamp: 'cwp 10/15/2006 18:50'!
execute
	| pattern class |
	pattern := OBTextRequest 
					prompt: 'Please type the name or fragment to look for' 
					template: ''.
	pattern ifNil: [^self].
	class := self findClassIn: target environment pattern: pattern.
	class ifNotNil: [self select: class asNode with: requestor announcer].! !

!OBCmdFindClass methodsFor: 'private' stamp: 'dc 5/17/2007 12:17'!
findClassIn: anEnvironment pattern: pattern
	| classNames className toMatch potentialClassNames |
	toMatch := (pattern copyWithout: $.) asLowercase.
	potentialClassNames := self allBehaviorsIn: anEnvironment.
	classNames := (pattern endsWith: '.')
					ifTrue: [potentialClassNames
							select: [:nm | nm asLowercase = toMatch]]
					ifFalse: [potentialClassNames
							select: [:n | n includesSubstring: toMatch caseSensitive: false]].
	classNames size = 0 ifTrue: [^ nil].
	classNames size = 1 ifTrue: [^ anEnvironment at: classNames first asSymbol].
	className := self userSelectionOf: classNames for: toMatch.
	^ className ifNotNil: [anEnvironment at: className asSymbol]
! !

!OBCmdFindClass methodsFor: 'accessing' stamp: 'cwp 10/9/2006 15:06'!
group
	^ #class! !

!OBCmdFindClass methodsFor: 'accessing' stamp: 'lr 6/9/2007 10:00'!
icon
	^ #findIcon! !

!OBCmdFindClass methodsFor: 'testing' stamp: 'cwp 10/9/2006 13:36'!
isActive
	^ target isEnvironment! !

!OBCmdFindClass methodsFor: 'accessing' stamp: 'cwp 10/9/2006 13:39'!
keystroke
	^ $f! !

!OBCmdFindClass methodsFor: 'accessing' stamp: 'cwp 10/9/2006 13:34'!
label
	^ 'find class...'! !

!OBCmdFindClass methodsFor: 'private' stamp: 'cwp 10/9/2006 14:24'!
userSelectionOf: classNames for: toMatch 
	| exactMatch labels lines |
	exactMatch := classNames detect: [:ea | ea asLowercase = toMatch] ifNone: [nil].
	exactMatch 
		ifNil: [labels _ classNames. lines _ #()]
		ifNotNil: [labels _ classNames copyWithFirst: exactMatch. lines _ #(1)].
	^ OBChoiceRequest labels: labels lines: lines.! !

OBCommand subclass: #OBCmdFindMethod
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OB-Standard-Commands'!

!OBCmdFindMethod methodsFor: 'as yet unclassified' stamp: 'cwp 10/31/2006 19:48'!
execute
	| selectors selection method |
	selectors := target theClass selectors asSortedArray.
	selectors isEmpty ifTrue: [^nil].
	selection := OBChoiceRequest labels: selectors.
	selection ifNotNil: 
		[method _ OBMethodNode on: selection inClass: target theClass.
		self select: method with: requestor announcer].! !

!OBCmdFindMethod methodsFor: 'as yet unclassified' stamp: 'cwp 6/8/2007 20:37'!
icon
	^ #findIcon! !

!OBCmdFindMethod methodsFor: 'as yet unclassified' stamp: 'cwp 10/16/2006 23:56'!
isActive
	^ (target isKindOf: OBClassNode) ! !

!OBCmdFindMethod methodsFor: 'as yet unclassified' stamp: 'cwp 11/3/2006 00:26'!
label
	^ 'find method...'! !

OBCommand subclass: #OBCmdLimitProtocol
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OB-Standard-Commands'!

!OBCmdLimitProtocol methodsFor: 'execution' stamp: 'DM 5/23/2007 14:49'!
execute
	| menu |
	menu := (MenuMorph entitled: self menuString) defaultTarget: self.
	menu
		add: (self menuLabelForClass: target theClass)
		selector: #limitClass:
		argument: target theClass;
		addLine.
	target theClass allSuperclassesDo: 
			[:each | 
			menu 
				add: (self menuLabelForClass: each)
				selector: #limitClass:
				argument: each].
	menu popUpInWorld! !

!OBCmdLimitProtocol methodsFor: 'accessing' stamp: 'DM 5/21/2007 14:23'!
group
	^ #general! !

!OBCmdLimitProtocol methodsFor: 'testing' stamp: 'DM 5/23/2007 18:51'!
isActive
	^ target theClass ~= ProtoObject and: [requestor isSelected: target]! !

!OBCmdLimitProtocol methodsFor: 'limit class' stamp: 'DM 5/22/2007 02:53'!
isLimitClass: aClass
	^ self limitClass = aClass.! !

!OBCmdLimitProtocol methodsFor: 'accessing' stamp: 'DM 5/23/2007 15:26'!
label
	^ requestor browser theClass = ProtoObject 
		ifTrue: ['limit']
		ifFalse: ['limit: ' , self limitClass name]! !

!OBCmdLimitProtocol methodsFor: 'limit class' stamp: 'DM 5/22/2007 18:41'!
limitClass
	^ requestor browser limitClass! !

!OBCmdLimitProtocol methodsFor: 'limit class' stamp: 'DM 5/31/2007 14:59'!
limitClass: aClass 
	requestor 
		announce: (OBNodeChanged node: aClass);
		announce: (OBRefreshRequired new)! !

!OBCmdLimitProtocol methodsFor: 'private' stamp: 'DM 5/22/2007 04:03'!
menuLabelForClass: aClass 
	| content |
	content := (self isLimitClass: aClass) 
				ifTrue: [aClass name asText addAttribute: TextEmphasis bold]
				ifFalse: [aClass name].
	^ content! !

!OBCmdLimitProtocol methodsFor: 'private' stamp: 'DM 5/21/2007 14:50'!
menuString

^ 'Show only methods implemented at or above...'! !

!OBCmdLimitProtocol methodsFor: 'testing' stamp: 'DM 5/21/2007 14:23'!
wantsButton
^ true! !

OBCommand subclass: #OBCmdMoveToPackage
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OB-Standard-Commands'!

!OBCmdMoveToPackage methodsFor: 'as yet unclassified' stamp: 'cwp 12/11/2006 20:33'!
execute
	| packagesNames selection packages |
	packages := PackageOrganizer default packages 
					asSortedCollection: [:a :b | a packageName <= b packageName].
	packagesNames := packages collect: [:ea | ea packageName].
	selection := OBChoiceRequest prompt: nil labels: packagesNames values: packages.
	selection ifNotNil: [selection addMethod: target reference].! !

!OBCmdMoveToPackage methodsFor: 'as yet unclassified' stamp: 'cwp 12/11/2006 20:19'!
isActive
	^ (target isKindOf: OBMethodNode) and: [requestor isSelected: target]! !

!OBCmdMoveToPackage methodsFor: 'as yet unclassified' stamp: 'cwp 12/11/2006 21:33'!
keystroke
	^ $p! !

!OBCmdMoveToPackage methodsFor: 'as yet unclassified' stamp: 'cwp 12/11/2006 20:13'!
label
	^ 'move to package...'! !

OBCommand subclass: #OBCmdRemoveCat
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OB-Standard-Commands'!

!OBCmdRemoveCat methodsFor: 'as yet unclassified' stamp: 'cwp 10/15/2006 18:52'!
execute
	| list choice |
	list := target container organization listAtCategoryNamed: target name.
	list isEmpty ifTrue: [^ target remove].
	choice := OBConfirmationRequest prompt: 'Are you sure you want to
remove this category 
and all its elements?' confirm: 'Remove'.
	choice ifTrue: [^ target remove]! !

!OBCmdRemoveCat methodsFor: 'as yet unclassified' stamp: 'cwp 10/9/2006 15:07'!
group
	^ #category! !

!OBCmdRemoveCat methodsFor: 'as yet unclassified' stamp: 'lr 6/9/2007 09:59'!
icon
	^ #deleteIcon! !

!OBCmdRemoveCat methodsFor: 'as yet unclassified' stamp: 'cwp 10/9/2006 16:06'!
isActive
	^ target isCategory and: [requestor isSelected: target]! !

!OBCmdRemoveCat methodsFor: 'as yet unclassified' stamp: 'cwp 9/28/2006 08:37'!
keystroke
	^ $x! !

!OBCmdRemoveCat methodsFor: 'as yet unclassified' stamp: 'cwp 9/28/2006 08:37'!
label
	^ 'remove...'! !

OBCommand subclass: #OBCmdRemoveClass
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OB-Standard-Commands'!

!OBCmdRemoveClass methodsFor: 'as yet unclassified' stamp: 'cwp 11/3/2006 00:13'!
execute
	(OBConfirmationRequest
		prompt: 'Are you certain that you 
want to REMOVE the class ', target theNonMetaClassName, ' from the system?'
		confirm: 'Remove')
			ifTrue: [target theNonMetaClass removeFromSystem.
					target signalDeletion]! !

!OBCmdRemoveClass methodsFor: 'as yet unclassified' stamp: 'cwp 11/3/2006 00:15'!
group
	^ #identity! !

!OBCmdRemoveClass methodsFor: 'as yet unclassified' stamp: 'cwp 6/8/2007 20:37'!
icon
	^ #deleteIcon! !

!OBCmdRemoveClass methodsFor: 'as yet unclassified' stamp: 'cwp 11/3/2006 00:11'!
isActive
	^ (target isKindOf: OBClassNode) and: [requestor isSelected: target]! !

!OBCmdRemoveClass methodsFor: 'as yet unclassified' stamp: 'cwp 11/3/2006 00:17'!
keystroke
	^ $x! !

!OBCmdRemoveClass methodsFor: 'as yet unclassified' stamp: 'cwp 11/3/2006 00:11'!
label
	^ 'remove...'! !

OBCommand subclass: #OBCmdRemoveMethod
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OB-Standard-Commands'!

!OBCmdRemoveMethod methodsFor: 'private' stamp: 'cwp 12/9/2006 20:37'!
browseSenders
	OBSendersBrowser browseRoot: target! !

!OBCmdRemoveMethod methodsFor: 'private' stamp: 'cwp 12/9/2006 19:48'!
doRemove
	target theClass removeSelector: target selector.
	requestor announce: (OBNodeDeleted node: target)! !

!OBCmdRemoveMethod methodsFor: 'command' stamp: 'cwp 12/9/2006 20:41'!
execute
	| senders choice |
	senders _ SystemNavigation default allCallsOn: target selector.
	senders isEmpty ifTrue: [^ self doRemove].
	choice _ OBChoiceRequest
				prompt: 'This message has ', senders size asString, ' senders.'
				labels: #('Remove it' 
						'Remove, then browse senders' 
						'Don''t remove, but show me those senders' 
						'Forget it -- do nothing -- sorry I asked')
				values: #(doRemove removeAndBrowse browseSenders nil).
	choice ifNotNil: [^ self perform: choice]
! !

!OBCmdRemoveMethod methodsFor: 'command' stamp: 'lr 6/9/2007 09:59'!
icon
	^ #deleteIcon! !

!OBCmdRemoveMethod methodsFor: 'command' stamp: 'cwp 12/8/2006 23:27'!
isActive
	^ (target isKindOf: OBMethodNode) and: [requestor isSelected: target]! !

!OBCmdRemoveMethod methodsFor: 'command' stamp: 'cwp 12/8/2006 23:18'!
keystroke
	^ $x! !

!OBCmdRemoveMethod methodsFor: 'command' stamp: 'cwp 12/8/2006 23:17'!
label
	^ 'remove...'! !

!OBCmdRemoveMethod methodsFor: 'private' stamp: 'cwp 12/9/2006 20:37'!
removeAndBrowse
	self browseSenders.
	self doRemove! !

OBCommand subclass: #OBCmdRenameCat
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OB-Standard-Commands'!

!OBCmdRenameCat methodsFor: 'as yet unclassified' stamp: 'cwp 10/15/2006 18:53'!
execute
	| category |
	category := OBTextRequest
				prompt: 'Please type new category name' 
				template: target name.
	category ifNotNil:	
		[target container organization renameCategory: target name toBe: category.
		target signalChanged]! !

!OBCmdRenameCat methodsFor: 'as yet unclassified' stamp: 'cwp 10/9/2006 15:07'!
group
	^ #category! !

!OBCmdRenameCat methodsFor: 'as yet unclassified' stamp: 'cwp 10/9/2006 16:06'!
isActive
	^ target isCategory and: [requestor isSelected: target]! !

!OBCmdRenameCat methodsFor: 'as yet unclassified' stamp: 'cwp 10/6/2006 21:16'!
label
	^ 'rename category...'! !

OBCommand subclass: #OBCmdRenameClass
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OB-Standard-Commands'!

!OBCmdRenameClass methodsFor: 'as yet unclassified' stamp: 'cwp 11/2/2006 23:47'!
browseObsoleteRefs: aClassNode as: oldName 
	| binding |
	binding := aClassNode theNonMetaClass environment 
				associationAt: aClassNode theNonMetaClass name.
	(SystemNavigation default allCallsOn: binding) isEmpty 
		ifFalse: [OBReferencesBrowser
					browseRoot: aClassNode
					title: 'Obsolete references to']! !

!OBCmdRenameClass methodsFor: 'as yet unclassified' stamp: 'cwp 11/2/2006 23:21'!
execute
	| newName |
	newName := OBTextRequest
					prompt: 'Please type new class name' 
					template: target theNonMetaClassName asString.
	newName ifNotNil:	[ | oldName |
						oldName := target theNonMetaClass name.
						target theNonMetaClass environment 
							renameClassNamed: oldName
							as: newName asSymbol.
						self browseObsoleteRefs: target as: oldName].
	! !

!OBCmdRenameClass methodsFor: 'as yet unclassified' stamp: 'cwp 11/2/2006 23:58'!
group
	^ #identity! !

!OBCmdRenameClass methodsFor: 'as yet unclassified' stamp: 'cwp 11/2/2006 23:01'!
isActive
	^ (target isKindOf: OBClassNode)
		and: [requestor isSelected: target]! !

!OBCmdRenameClass methodsFor: 'as yet unclassified' stamp: 'cwp 11/2/2006 22:47'!
label
	^ 'rename...'! !

OBCommand subclass: #OBCmdRevertMethod
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OB-Standard-Commands'!

!OBCmdRevertMethod methodsFor: 'as yet unclassified' stamp: 'lr 4/25/2007 09:31'!
buttonLabel
	^ 'revert'! !

!OBCmdRevertMethod methodsFor: 'as yet unclassified' stamp: 'cwp 10/15/2006 18:54'!
execute
	target version fileIn! !

!OBCmdRevertMethod methodsFor: 'as yet unclassified' stamp: 'cwp 10/9/2006 18:30'!
isActive
	^ (target isKindOf: OBMethodVersionNode) and: [requestor isSelected: target]! !

!OBCmdRevertMethod methodsFor: 'as yet unclassified' stamp: 'cwp 10/9/2006 18:26'!
keystroke
	^ $r! !

!OBCmdRevertMethod methodsFor: 'as yet unclassified' stamp: 'cwp 10/9/2006 18:24'!
label
	^ 'revert to version'! !

!OBCmdRevertMethod methodsFor: 'as yet unclassified' stamp: 'lr 4/25/2007 09:38'!
wantsButton
	^ target isKindOf: OBMethodVersionNode! !

OBCommand subclass: #OBCmdSubclassTemplate
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OB-Standard-Commands'!

!OBCmdSubclassTemplate methodsFor: 'as yet unclassified' stamp: 'cwp 12/7/2006 23:35'!
execute
	| class definition |
	class _ target theNonMetaClass.
	definition _ (OBClassDefinition 
					environment: class environment 
					template: (Class 
								templateForSubclassOf: class 
								category: class category)).
	requestor announce: (OBDefinitionChanged definition: definition)! !

!OBCmdSubclassTemplate methodsFor: 'as yet unclassified' stamp: 'cwp 12/7/2006 23:14'!
isActive
	^ (target isKindOf: OBClassNode) and: [requestor isSelected: target]! !

!OBCmdSubclassTemplate methodsFor: 'as yet unclassified' stamp: 'cwp 12/7/2006 23:04'!
label
	^ 'subclass template'! !

!SystemOrganizer methodsFor: '*ob-standard-testing' stamp: 'cwp 9/18/2004 23:58'!
isClassOrganizer
	^ false! !
OBCodeBrowser initialize!
OBSearchBrowser initialize!
OBSystemBrowser initialize!
OBMercuryPanel initialize!
OBSystemBrowserAdaptor initialize!
